;****************************************************************************
;
;  Filename:      roboear.asm
;  Date:          24-June-2018
;  File Version:  1.06
;  Author:        Brian Beard
;  Company:       Lucid Technologies, lucidtechnologies.info
;  Description:   Robot ear project
;
;****************************************************************************
;
;  Files Required:  P16F1847.INC
;
;****************************************************************************
;----------------------------------------------------------------------------
; PIC16F1847 PIN USAGE
;               	-------- Interface -------
; -- Pin --   Pullup	-- UART --	-- SPI --
; 17   RA0	N	ADC1   In	ADC1   In
; 18   RA1	N	ADC2   In	ADC2   In
;  1   RA2	N	ADC3   In	ADC3   In
;  2   RA3	Y	BAUD0  In	CKE    In
;  3   RA4	Y	BAUD1  In	CKP    In
;  4   RA5	Y	MCLR   In	MCLR   In
; 15   RA6	N	ADC_CS Out	ADC_CS Out
; 16   RA7	N	ADC_CK Out	ADC_CK Out
;  6   RB0	Y	nc     In	BRDY   Out
;  7   RB1	Y	RX     In	SDI    In
;  8   RB2	N	TX     Out	SDO    Out
;  9   RB3	N	(CTS)  Out	BUSY   Out
; 10   RB4	Y	(RTS)  In	SCK    In
; 11   RB5	Y	nc     In	SS\    In
; 12   RB6	Y	nc     In	nc     In
; 13   RB7	Y	MODE   In	MODE   In
; 14   Vdd	na	+5V		+5V	
;  5   Vss	na	Ground		Ground	
;
; DTE (Host) asserts RTR/RTS when ready to receive data from Roboear.
; DCE (Roboear) asserts CTS/BUSY when ready to receive command from Host.
;****************************************************************************
; PROGRAM MEMORY
; The PIC16F1847 has 8k (2000h) of FLASH program memory in four 2k pages.
; Page0=0000:07FF, Page1=0800:0FFF, PAGE2=1000h-17FFh, PAGE3=1800h-1FFFh 
;
;****************************************************************************
; RANDOM ACCESS MEMORY (REGISTERS)
; The PIC16F1847 has 1K (400h) bytes of RAM (General Purpose Registers, GPR)
; in 32 Banks. Each bank is 128 bytes, including Special Function Registers.
; The bottom 12 bytes of each Bank are the CORE SFRs, the top 16 bytes of 
; each Bank are the COMMON RAM.
;
;****************************************************************************
; The File Select Register pairs (FSR0H:FSR0L and FSR1H:FSR1L) allow direct
; linear access to RAM and flash program memory.
;   0000:0FFF  traditional SFR and GPR access
;   2000:23FF  linear RAM (GPR) access
;   8000:9FFF  linear flash program memory access
;
;****************************************************************************
; EEPROM DATA MEMORY VARIABLE DEFINITIONS
; The on-chip EEPROM is only 256 bytes.
; For program access purposes, the address range is 00:7F.
; For defined data from the assembler *.HEX file, the address is 2100:217F.
;
;****************************************************************************
; CLOCK & TIMER DATA
Fosc	EQU		D'16000000'	; Oscillator frequency
Fop	EQU		Fosc/4		; Operating frequency
;
; For USART asynchonous mode with Fosc = 16 MHz:
; SYNC=0, BRGH=1, BRG16=0, SPBRGH=0 ; 8-bit asynch mode, divide by 16
;                 SPBRGL    Baud Rate    Error
BD4800    equ     D'207'   ; 4800        0.16%
BD9600    equ     D'103'   ; 9600        0.16%
BD19200   equ     D'51'    ; 19200       0.16%
BD38400   equ     D'25'    ; 38400       0.16%
BD76800   equ     D'12'    ; 76800       0.16%
;
;****************************************************************************
; CONSTANT DEFINITIONS 
;
; ASCII special characters
ACK	EQU	0x06      ; acknowledge
BELL	EQU	0x07      ; bell
BS	EQU	0x08      ; back space
TAB	EQU	0x09      ; tab
LF	EQU	0x0A      ; line feed
CR	EQU	0x0D      ; carriage return
NAK	EQU	0x15      ; not acknowledge
SYN	EQU	0x16      ; sync
ESC	EQU	0x1B      ; escape
;
;****************************************************************************
;
	list      p=16F1847         ; list directive to define processor
	#include <p16F1847.inc>     ; processor specific definitions
	errorlevel  -302            ; suppress message 302 from list file
;
;Program Configuration Register 1
;FCMEM      Fail safe clock			DISABLE	0
;IESO       Internal-external switchover	DISABLE	0
;CLKOUTEN\  External clock out 			DISABLE	1
;BOREN      Brown out reset			ENABLE	11
;CPD\       Data memory code protection		DISABLE	1
;CP\        Program memory code protection	DISABLE	1
;MCLRE      Master clear 			ENABLE	1
;PWRTE\     Power up timer			ENABLE	0
;WDTE       Watch dog timer			DISABLE	00
;FOSC       Select INTOSC				100
	__CONFIG    _CONFIG1, B'00111111000100'
;
;Program Configuration Register 2
;LVP        Low voltage programming		DISABLE	0
;DEBUG\     In-circuit debug			DISABLE	1
;bit 11     Unimplemented				1
;BORV       Brown out voltage = high			0
;STVREN     Stack over/under-flow reset		DISABLE	0
;PLLEN      Phase locked loop			DISABLE	0
;bits 2:7   Unimplemented				111111
;WRT        Flash memory self-write protect	ENABLE	00
	__CONFIG    _CONFIG2, B'01100011111100'
;
;****************************************************************************
; VARIABLE DEFINITIONS 
	CBLOCK	0x020           ; define constants in bank 0 GPR
; Variables used in subroutines
dly_cnt      ; delay subroutine
hex_asc      ; temporary hex to ascii, or vice versa
asc_msn      ; most significant nibble, ascii
asc_lsn      ; least significant nibble, ascii
bit_cnt      ; bit counter
byt_cnt      ; byte counter
dig_cnt      ; digit counter
lin_cnt      ; line counter
off_set      ; offset pointer
rx_buf       ; receive buffer
tx_buf       ; transmit buffer
tmp_1        ; subroutine temporary data location 1
tmp_2        ; subroutine temporary data location 2
c_hold       ; Carry bit hold
s_hold       ; shift data hold 
adc_1        ; temporary location for unsigned binary ADC1 data
adc_2        ; temporary location for unsigned binary ADC2 data
adc_3        ; temporary location for unsigned binary ADC3 data
gnd_1        ; analog ground unsigned binary value for channel 1
gnd_2        ; analog ground unsigned binary value for channel 2
gnd_3        ; analog ground unsigned binary value for channel 3
rng_1        ; unsigned binary range of values for channel 1
rng_2        ; unsigned binary range of values for channel 2
rng_3        ; unsigned binary range of values for channel 3
brng_a       ; bearing angle computed from amplitude variations
brng_x       ; bearing angle computed from cross-correlations
lag_12       ; max lag index for baseline 1-2
lag_23       ; max lag index for baseline 2-3
lag_31       ; max lag index for baseline 3-1
xc_lc        ; cross-correlation lag counter
xc_sc        ; cross-correlation sum counter
xc_nc        ; cross-correlation negatives counter
ACa_2        ; 24-bit Accumulator a, lsb+2, ms-byte
ACa_1        ; 24-bit Accumulator a, lsb+1
ACa_0        ; 24-bit Accumulator a, ls-byte
ACb_2        ; 24-bit Accumulator b, lsb+2, ms-byte
ACb_1        ; 24-bit Accumulator b, lsb+1
ACb_0        ; 24-bit Accumulator b, ls-byte
ACc_2        ; 24-bit Accumulator c, lsb+2, ms-byte
ACc_1        ; 24-bit Accumulator c, lsb+1
ACc_0        ; 24-bit Accumulator c, ls-byte
ACd_2        ; 24-bit Accumulator d, lsb+2, ms-byte
ACd_1        ; 24-bit Accumulator d, lsb+1
ACd_0        ; 24-bit Accumulator d, ls-byte
ACx_2        ; 24-bit Accumulator x, lsb+2, ms-byte
ACx_1        ; 24-bit Accumulator x, lsb+1
ACx_0        ; 24-bit Accumulator x, ls-byte
ACy_2        ; 24-bit Accumulator y, lsb+2, ms-byte
ACy_1        ; 24-bit Accumulator y, lsb+1
ACy_0        ; 24-bit Accumulator y, ls-byte
ACz_2        ; 24-bit Accumulator z, lsb+2, ms-byte
ACz_1        ; 24-bit Accumulator z, lsb+1
ACz_0        ; 24-bit Accumulator z, ls-byte
bcd_7        ; 10^7, tens of millions
bcd_6        ; 10^6, millions
bcd_5        ; 10^5, hundreds of thousands
bcd_4        ; 10^4, tens of thousands
bcd_3        ; 10^3, thousands
bcd_2        ; 10^2, hundreds
bcd_1        ; 10^1, tens
bcd_0        ; 10^0, units
	ENDC
;
	CBLOCK	0x70            ; Common Registers = 070:07F 
; Variables for main program
mode		; mode flags register
freg		; flags register
csetb		; comm settings bits: baud rate, clock polarity
min_rng		; minimum signal range
cnt01		; common counter 1
tmp01		; common temporary location 
indx1		; index into amplitude bearing table
indx2           ; index into cross-correlation bearing table
	ENDC
; Bit definitions for mode register (mode)
rdf	equ	0     ; raw data format: 1 = Hex, 0 = decimal
baf	equ	1     ; bearing angle format: 1 = BAM, 0 = decimal
ccm	equ	2     ; computer control mode: 1 = true, 0 = false
hcm	equ	7     ; host communications mode: 1 = UART, 0 = SPI
; Bit definitions for flag register (freg)
rok	equ	0     ; ranges > min_rng: 1 = true, 0 = false
r08	equ	1     ; ready 8-bit data:  1 = true, 0 = false
r10	equ	2     ; ready 10-bit data:  1 = true, 0 = false
rcb	equ	3     ; ready computed bearings: 1 = true, 0 = false
;
; Define linear address buffers for 8-bit data
; Each channel needs 127 bytes of storage
c1dat	equ	0x2070         ; 128 bytes to 20EF
c2dat	equ	0x20F0         ; 128 bytes to 216F
c3dat	equ	0x2170         ; 128 bytes to 21EF
; Acquisition buffer, 127x4 = 508 bytes of the GPR from banks 9-12
abuff	equ	0x21F0         ; 511 bytes to 23EF
;
;****************************************************************************
;*****      Macros.
;****************************************************************************
;
;
;****************************************************************************
;*****      Start of program memory, ET and INTerrupt vectors.
;****************************************************************************
;
	ORG     0x0000            ; processor reset vector
	nop                       ; required for ICD				
	clrf    PCLATH            ; ensure page bits are cleared
	goto    MAIN              ; go to beginning of program
;
	ORG     0x0004            ; interrupt vector location
; enhanced mid-range PICs have automatic context saving
	goto    MAIN              ; go to beginning of program
; isr code can go here or be located as a call subroutine elsewhere
	retfie                    ; return from interrupt
;
;****************************************************************************
;*****      Interrupt service routines.
;****************************************************************************

; 

;============================================================================

;****************************************************************************
;*****      Subroutines.
;****************************************************************************
; All subroutine names (labels) begin with a capital letter.
; Subroutine's internal variable are in bank0 and return with bank0 selected.
;============================================================================
;
;============================================================================
; Delay subroutine.  Load W with the delay value before calling,
; FFh >= W >= 00.  The total delay including the CALL and RETURN to the 
; line following the call, is 4*(W)+8 clock cycles.  W = 0 on returned.
; Call "Delay" for delay value in W.
; Call "Dlymax" for delay value of FFh, 256 usec for Fosc = 16 MHz
; Registers modified: BSR, dly_cnt
; Subroutines called: none
; Error code: none
Dlymax	movlw	0xFF          ; 1 cycle, maximum delay value
Delay	movlb	0             ; 1 cycle, 0 --> BSR, select bank 0
	movwf	dly_cnt       ; 1 cycle, store delay value
	incf	dly_cnt,f     ; 1 cycle, allows delay(0)
dly10	nop                   ; 1 cycle
	decfsz	dly_cnt,f     ; 1 cycle, (dlycnt) - 1 --> (dlycnt)
	goto	dly10         ; 2 cycles, again if > 0
dly99	retlw	0             ; 2 cycles, bank 0
;============================================================================
; Convert the low nibble of W to ascii character returned in W.
; Registers modified: BSR, hex_asc
; Subroutines called: none
; Error code: none
Hex1asc
	movlb	0            ; 0 --> BSR, select bank 0
	andlw	0x0F         ; mask off upper nibble
	addlw	D'48'        ; add ascii '0'
	movwf	hex_asc      ; save in temporary variable
	sublw	D'57'        ; 57 - W --> W, carry=1 if 0-9
	clrw                 ; 00 --> W, carry unaffected
	btfss	STATUS,C     ; skip if 0-9
	movlw	D'7'         ; 7 --> W, if A-F
	addwf	hex_asc,W    ; (hex_asc) + W --> W
	return	             ; bank 0
;============================================================================
; Convert the Hex value in W into its two character ascii representation.
; Most-significant nibble's ascii character --> (asc_msn),
; least-significant nibble's ascii character --> (asc_lsn).
; Registers modified: BSR, asc_msn, asc_lsn
; Subroutines called: Hex1asc
; Error code: none
Hex2asc
	movlb	0            ; 0 --> BSR, select bank 0
	movwf	asc_msn      ; msn:lsn --> (asc_msn)
	call	Hex1asc      ; convert the ls-nibble
	movwf	asc_lsn      ; save ls-nibble ascii
	swapf	asc_msn,w    ; lsn:msn --> W
	call	Hex1asc      ; convert the ms-nibble
	movwf	asc_msn      ; save ms-nibble ascii
	return	             ; bank 0
;============================================================================
; Single byte unsigned multiplication, adapted from AN526.
; 8-bit by 8-bit multiply with 16-bit product. (ACb_0)*(ACa_0) --> (ACc) 
; Preload ACa_0 and ACb_0, no need to clear upper bytes of ACa and ACb.
; Registers modified: BSR, bit_cnt, ACa, ACb, ACc
; Subroutines called: none
; Error code: none
S_mult
	movlb	0             ; Select bank 0
	movlw   D'8'          ; for 8 shifts
	movwf   bit_cnt
	clrf	ACa_1        ; clear ACa_1
	clrf	ACc_2        ; clear ACc_2
	clrf    ACc_1        ; clear ACc_1
	clrf    ACc_0        ; clear ACc_0
mloop   
	rrf     ACb_0,f      ;rotate ACb right
	btfss   STATUS,C      ;need to add?
	goto	no_add        ;no addition necessary
	movf    ACa_0,w      ;Addition ( ACa + ACc -> ACc )
	addwf   ACc_0,f       ;add LSB 
	movf    ACa_1,w
	addwfc	ACc_1,f       ;add MSB with Carry
no_add
	lslf	ACa_0,f
	rlf     ACa_1,f
	decfsz  bit_cnt,f     ;loop until all bits checked
	goto    mloop
	retlw   0             ; return W=0, bank 0
;============================================================================
; Triple Precision (24-bit) Addition & Subtraction, adapted from AN526.
; Signed three-byte (24-bit) binary accumulators, -8,388,607 to 8,388,607.
; ACa_2 and ACb_2 are the MS-Bytes, ACa_0 and ACb_0 are the LS-Bytes.
; Addition (T_add):    (ACb)+(ACa) --> (ACb), (ACa) is unchanged
; Subtraction (T_sub): (ACb)-(ACa) --> (ACb), (ACa) is negated
T_sub
	call    Neg_ACa       ;First negate ACa, then add
T_add
	movlb	0             ;Select bank 0
	movf    ACa_0,w
	addwf   ACb_0,f       ;Add lsb
	btfss   STATUS,C      ;Test for carry
	goto	qa_2          ; go if no carry
	incf    ACb_1,f        ; if C=1, add carry to lsb+1
	btfsc	STATUS,Z      ;Test for carry ripple (FF --> 00)
	incf	ACb_2,f        ; if Z=1, add carry to lsb+2, msb
qa_2	movf    ACa_1,w
	addwf   ACb_1,f        ;Add lsb+1
	btfsc   STATUS,C      ;Test for carry
	incf	ACb_2,f        ; if C=1, add carry to lsb+2, msb
qa_4	movf    ACa_2,w
	addwf   ACb_2,f       ;Add lsb+2, msb
	retlw   0             ; return W=0, bank 0
;============================================================================
; Negate the value in the 24-bit accumulator A.
Neg_ACa
	movlb	0             ;Select bank 0
	comf    ACa_0,f       ; complement (ACa)
	comf    ACa_1,f
	comf    ACa_2,f
	incf    ACa_0,f       ; add one
	btfss   STATUS,Z
	retlw	0
	incf    ACa_1,f
	btfss   STATUS,Z
	retlw	0
	incf	ACa_2,f
	retlw   0             ; return W=0, bank 0
;============================================================================
; Negate the value in the 24-bit accumulator X.
Neg_ACx
	movlb	0             ;Select bank 0
	comf    ACx_0,f       ; complement (ACx)
	comf    ACx_1,f
	comf    ACx_2,f
	incf    ACx_0,f       ; add one
	btfss   STATUS,Z
	retlw	0
	incf    ACx_1,f
	btfss   STATUS,Z
	retlw	0
	incf	ACx_2,f
	retlw   0             ; return W=0, bank 0
;============================================================================
; Negate the value in the 24-bit accumulator Y.
Neg_ACy
	movlb	0             ;Select bank 0
	comf    ACy_0,f       ; complement (ACy)
	comf    ACy_1,f
	comf    ACy_2,f
	incf    ACy_0,f       ; add one
	btfss   STATUS,Z
	retlw	0
	incf    ACy_1,f
	btfss   STATUS,Z
	retlw	0
	incf	ACy_2,f
	retlw   0             ; return W=0, bank 0
;============================================================================
; Triple Precision (24-bit) unsigned Binary to BCD conversion.
; The 24-bit binary number in ACb is converted to a eight digit unpacked 
; BCD number in bcd_7 to bcd_0. The upper nibble of each bcd digit is zero.
; Registers modified: BSR, FSR0, bit_cnt, dig_cnt, ACb
; Subroutines called: none
; Error code: none
T_b2bcd
	movlb	0            ;Select bank 0
	clrf	bcd_7        ;Clear all bcd digits
	clrf	bcd_6
	clrf	bcd_5
	clrf	bcd_4
	clrf	bcd_3
	clrf	bcd_2
	clrf	bcd_1
	clrf	bcd_0
	movlw	D'24'        ;Outer loop, binary bits
	movwf	bit_cnt       ; bit counter
b2bcd1
	rlf	ACb_0,f       ;Shift 24-bit accumulator
	rlf	ACb_1,f       ; left to put
	rlf	ACb_2,f       ;  ms-bit into Carry
	movlw	bcd_0         ;Point to address of least 
	movwf	FSR0L         ; significant bcd digit
	clrf	FSR0H
	movlw	D'8'         ;Inner loop, BCD digits
	movwf	dig_cnt       ; digit counter
b2bcd2
	rlf	INDF0,f      ;Shift Carry into bcd digit
	movlw	D'10'        ;Subtract ten from digit then
	subwf	INDF0,w      ; check and adjust for decimal overflow
	btfsc   STATUS,C     ;If Carry = 1 (result >= 0)
	movwf	INDF0        ; adjust for decimal overflow
	decf	FSR0L,f      ;Point to next bcd digit
	decfsz	dig_cnt,f    ;Decrement digit counter
	goto	b2bcd2       ; - go if dig_cnt > 0
	decfsz	bit_cnt,f    ;Decrement bit counter
	goto	b2bcd1       ; - go if bit_cnt > 0
	retlw   0             ; return W=0, bank 0
;============================================================================
; Triple Precision (24-bit) unsigned BCD to Binary conversion.
; bcd_7 to bcd_0 comprise one 8-digit unpacked Binary-Coded-Decimal number.
; The upper nibble of each digit is zero (00008421). bcd_7 is the MS-Digit.
; ACb_2 to ACb_0 comprise a three-byte (24-bit) binary accumulator.
; ACb_2 is the MS-Byte, ACb_0 is the LS-byte. 
; The 8-digit BCD number is converted to a 24-bit binary number in ACb.
T_bcd2b
	movlb	0             ;Select bank 0
	movlw	D'24'         ;Outer loop, binary bits
	movwf	bit_cnt       ; bit counter
bcd2b1
	movlw	bcd_7         ;Set pointer for
	movwf	FSR0L         ; most significant digit
	clrf	FSR0H
	movlw	D'8'          ;Inner loop, BCD digits
	movwf	dig_cnt       ; digit counter
	movlw	0x05          ;Weight of next MS-Digit's carry bit
	clrf	c_hold        ;0 --> carry_hold
bcd2b2
	bcf	STATUS,0      ;Clear Carry bit
	rrf	INDF0,f       ;Right shift 0 into bcd digit
	rlf	c_hold,f      ;Carry --> c_hold,0
	btfsc	c_hold,1      ;Add five if carry from previous rotate is
	addwf	INDF0,f       ; set, else no change
	incf	FSR0L,f       ;Advance digit pointer
	decfsz	dig_cnt,f      ;Decrement digit counter,
	goto	bcd2b2        ; repeat if > 0
	rrf	c_hold,f      ;Last carry from digit rotate --> C 
	rrf	ACb_2,f        ;Ripple Carry bits from 
	rrf	ACb_1,f        ;  MSB to LSB of
	rrf	ACb_0,f        ;    24-bit binary accumulator
	decfsz	bit_cnt,f      ;Decrement bit counter,
	goto	bcd2b1        ; repeat if > 0
	retlw   0             ; return W=0, bank 0
;============================================================================
; Triple precision (24-bit) unsigned divide, adapted from AN526.
; 24-bit by 24-bit divide with 24-bit quotient and 24-bit remainder. 
; (ACb)/(ACa) --> (ACb), remainder in ACc.
; NOTE: Before calling this routine, the user should make sure that
;       the Numerator(ACb) is greater than Denominator(ACa). If
;       the case is not true, the user should scale either Numerator
;       or Denominator or both such that Numerator is greater than
;       the Denominator.
T_div
	movlb	0         ; Select bank 0
	movlw   D'24'     ; for 24 shifts
	movwf   bit_cnt
	movf    ACb_2,w    ;move dividend from ACb to ACd
	movwf   ACd_2
	movf    ACb_1,w
	movwf   ACd_1
	movf    ACb_0,w
	movwf   ACd_0
	clrf    ACb_2      ; clear ACb to hold quotient
	clrf    ACb_1
	clrf    ACb_0
	clrf    ACc_2      ; clear ACc to hold remainder
	clrf    ACc_1
	clrf    ACc_0
dloop
	bcf     STATUS,C
	rlf     ACd_0,f    ; Shift dividend (ACd) msb 
	rlf     ACd_1,f
	rlf     ACd_2,f
	rlf     ACc_0,f    ;  into remainder (ACc)
	rlf     ACc_1,f
	rlf     ACc_2,f
	movf	ACa_2,w    ; Test if remainder (ACc) >= divisor (ACa)
	subwf	ACc_2,w    ; ACc_2 - ACa_2 --> W
	btfss   STATUS,C   ; skip if ACc_2>=ACa_2, Carry=1  
	goto	d_40       ;  else ACc_2<ACa_2, shift 0 into quotient
	btfss   STATUS,Z   ; skip to next byte if ACc_2=ACa_2
        goto    d_20       ;  else ACc_2>ACa_2, subtract then sift 1 in
	movf	ACa_1,w    ; Test if remainder (ACc) >= divisor (ACa)
	subwf	ACc_1,w    ; ACc_1 - ACa_1 --> W
	btfss   STATUS,C   ; skip if ACc_1>=ACa_1, Carry=1  
	goto	d_40       ;  else ACc_1<ACa_1, shift 0 into quotient
	btfss   STATUS,Z   ; skip to next byte if ACc_1=ACa_1
        goto    d_20       ;  else ACc_1>ACa_1, subtract then sift 1 in
	movf	ACa_0,w    ; Test if remainder (ACc) >= divisor (ACa)
	subwf	ACc_0,w    ; ACc_0 - ACa_0 --> W
	btfss   STATUS,C   ; skip if ACc_2>=ACa_2, Carry=1  
	goto	d_40       ;  else ACc_2<ACa_2, shift 0 into quotient
d_20            ;  ACc >= ACa, subtract then shift 1 into quotient
	movf	ACa_0,w     ; Subtract divisor (ACa) from remainder (ACc)
	subwf	ACc_0,f     ; ACc_0 - ACa_0 --> ACc_0
	movf	ACa_1,w     ; next byte
	btfss   STATUS,C   ; Skip if last result positive or zero
	incfsz	ACa_1,w     ;  else subract one more from next byte
	subwf	ACc_1,f     ; ACc_1 - ACa_1 --> ACc_1
	movf	ACa_2,w     ; next byte
	btfss   STATUS,C   ; Skip if last result positive or zero
	incfsz	ACa_2,w     ;  else subract one more from next byte
	subwf	ACc_2,f     ; ACc_2 - ACa_2 --> ACc_2
	bsf     STATUS,C
        goto    d_60
d_40            ;  ACc < ACa, shift 0 into quotient
	bcf     STATUS,C
d_60            ; shift Carry into quotient (ACb) lsb
	rlf     ACb_0,f    ; Shift new lsb into quotient (ACb) 
	rlf     ACb_1,f
	rlf     ACb_2,f
	decfsz	bit_cnt,f    ;Next
	goto	dloop
	retlw   0            ; return W=0, bank 0
;============================================================================
; Copy accumulator B to accumulator D. ACb --> ACd.
; Registers modified: BSR, ACd
; Subroutines called: none
; Error code: none
Copy_b2d
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACb_2,w     ; ACb_2 --> W
	movwf	ACd_2       ;           W --> ACd_2
	movf	ACb_1,w     ; ACb_1 --> W
	movwf	ACd_1       ;           W --> ACd_1
	movf	ACb_0,w     ; ACb_0 --> W
	movwf	ACd_0       ;           W --> ACd_0
	return	            ; bank 0
;============================================================================
; Copy accumulator C to accumulator A. ACc --> ACa.
; Registers modified: BSR, ACa
; Subroutines called: none
; Error code: none
Copy_c2a
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACc_2,w     ; ACc_2 --> W
	movwf	ACa_2       ;           W --> ACa_2
	movf	ACc_1,w     ; ACc_1 --> W
	movwf	ACa_1       ;           W --> ACa_1
	movf	ACc_0,w     ; ACc_0 --> W
	movwf	ACa_0       ;           W --> ACa_0
	return	            ; bank 0
;============================================================================
; Copy accumulator C to accumulator B. ACc --> ACb.
; Registers modified: BSR, ACb
; Subroutines called: none
; Error code: none
Copy_c2b
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACc_2,w     ; ACc_2 --> W
	movwf	ACb_2       ;           W --> ACb_2
	movf	ACc_1,w     ; ACc_1 --> W
	movwf	ACb_1       ;           W --> ACb_1
	movf	ACc_0,w     ; ACc_0 --> W
	movwf	ACb_0       ;           W --> ACb_0
	return	            ; bank 0
;============================================================================
; Copy accumulator D to accumulator B. ACd --> ACb.
; Registers modified: BSR, ACb
; Subroutines called: none
; Error code: none
Copy_d2b
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACd_2,w     ; ACd_2 --> W
	movwf	ACb_2       ;           W --> ACb_2
	movf	ACd_1,w     ; ACd_1 --> W
	movwf	ACb_1       ;           W --> ACb_1
	movf	ACd_0,w     ; ACd_0 --> W
	movwf	ACb_0       ;           W --> ACb_0
	return	            ; bank 0
;============================================================================
; Copy accumulator D to accumulator Z. ACd --> ACz.
; Registers modified: BSR, ACz
; Subroutines called: none
; Error code: none
Copy_d2z
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACd_2,w     ; ACd_2 --> W
	movwf	ACz_2       ;           W --> ACz_2
	movf	ACd_1,w     ; ACd_1 --> W
	movwf	ACz_1       ;           W --> ACz_1
	movf	ACd_0,w     ; ACd_0 --> W
	movwf	ACz_0       ;           W --> ACz_0
	return	            ; bank 0
;============================================================================
; Copy accumulator X to accumulator A. ACx --> ACa.
; Registers modified: BSR, ACa
; Subroutines called: none
; Error code: none
Copy_x2a
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACx_2,w     ; ACx_2 --> W
	movwf	ACa_2       ;           W --> ACa_2
	movf	ACx_1,w     ; ACx_1 --> W
	movwf	ACa_1       ;           W --> ACa_1
	movf	ACx_0,w     ; ACx_0 --> W
	movwf	ACa_0       ;           W --> ACa_0
	return	            ; bank 0
;============================================================================
; Copy accumulator X to accumulator B. ACx --> ACb.
; Registers modified: BSR, ACa
; Subroutines called: none
; Error code: none
Copy_x2b
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACx_2,w     ; ACx_2 --> W
	movwf	ACb_2       ;           W --> ACb_2
	movf	ACx_1,w     ; ACx_1 --> W
	movwf	ACb_1       ;           W --> ACb_1
	movf	ACx_0,w     ; ACx_0 --> W
	movwf	ACb_0       ;           W --> ACb_0
	return	            ; bank 0
;============================================================================
; Copy accumulator Y to accumulator B. ACy --> ACb.
; Registers modified: BSR, ACb
; Subroutines called: none
; Error code: none
Copy_y2b
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACy_2,w     ; ACy_2 --> W
	movwf	ACb_2       ;           W --> ACb_2
	movf	ACy_1,w     ; ACy_1 --> W
	movwf	ACb_1       ;           W --> ACb_1
	movf	ACy_0,w     ; ACy_0 --> W
	movwf	ACb_0       ;           W --> ACb_0
	return	            ; bank 0
;============================================================================
; Copy accumulator Z to accumulator A. ACz --> ACa.
; Registers modified: BSR, ACa
; Subroutines called: none
; Error code: none
Copy_z2a
	movlb	0           ; 0 --> BSR, select bank 0
	movf	ACz_2,w     ; ACz_2 --> W
	movwf	ACa_2       ;           W --> ACa_2
	movf	ACz_1,w     ; ACz_1 --> W
	movwf	ACa_1       ;           W --> ACa_1
	movf	ACz_0,w     ; ACz_0 --> W
	movwf	ACa_0       ;           W --> ACa_0
	return	            ; bank 0
;============================================================================
; Clear any UART errors and empty receive fifo. 
; Registers modified: BSR, RCSTA
; Subroutines called: none
; Error code: none
UART_clr
	banksel	 RCREG         ; select bank 3
; two reads empties fifo and clears any FERR
        movf     RCREG,w       ; read first rx fifo byte
        movf     RCREG,w       ; read second rx fifo byte
; clearing CREN clears any OERR
        bcf      RCSTA,CREN    ; clear continuous rx enable bit
        bsf      RCSTA,CREN    ; reset continuous rx enable bit
	movlb    0             ; 0 --> BSR, select bank 0
	return	               ; bank 0
;============================================================================
; Receive one 7-bit ascii character byte from the host via the
; communications mode set by jumpers and stored in (mode). 
; The received byte is returned in W from the subroutine branched to.
; Registers modified: none
; Error code: none
Rxbyte
	btfss	mode,hcm      ; test host communications mode
	goto	SPI_rx        ; 0 = SPI mode,  RB7 jumpered to Gnd
	goto	UART_rx       ; 1 = UART mode, no jumpers
;============================================================================
; Receive a 7-bit ascii character byte from the UART, returned in W.
; This subroutine will loop until receive register (RCREG) is full.
; Registers modified: BSR, RCSTA
; Subroutines called: none
; Error code: W=0x80 if a framing error occurs.
UART_rx
	banksel	PIR1          ; select bank 0
rxb10	btfss	PIR1,RCIF     ; Test for RCREG empty (RCIF=0)
	bra	rxb10         ; - loop while RCREG empty (RCIF=0)
	banksel	RCSTA         ; select bank 3
	btfss	RCSTA,FERR    ; was there a framing error ?
	bra	rxb90         ; - go if no framing error	
	movf	RCREG,w       ; read and discard bad rx character
	movlb	0             ; 0 --> BSR, select bank 0
	retlw	0x80          ; return 0x80 in W if FERR=1
rxb90	movf	RCREG,w       ; read rx data register in bank 0
	andlw	0x7F          ; 7-bit ascii characters (0XXXXXXX)
	movlb	0             ; 0 --> BSR, select bank 0
	return	              ; return received byte in W
;============================================================================
; Receive a 7-bit ascii character byte via the SPI, returned in W.
; This subroutine will loop until SPI Slave Select is active (low),
; it will then wait for a new byte to be shifted in (BF=1).
; Registers modified: BSR, 
; Subroutines called: none
; Error code: none
SPI_rx
	movlb	4             ; select bank 4 for SSP registers
	movf	SSP1BUF,w     ;  read buffer to clear BF flag
	movlb	0             ; select bank 0 to read ports
SPIr_1
	btfsc	PORTB,5       ; loop until slave select (RB5)
	bra	SPIr_1        ;  goes low (active)
	movlb	4             ; select bank 4 for SSP registers
SPIr_2
	btfss	SSP1STAT,BF   ; loop until new byte shifted in and 
	bra	SPIr_2        ;  Buffer Full flag is set
	movf	SSP1BUF,w     ; read command byte, clears BF
	bcf	SSP1CON1,WCOL ; clear write collision bit
	andlw	0x7F          ; 7-bit ascii characters (0XXXXXXX)
	movlb	0             ; select bank 0
	return	              ; return received byte in W
;============================================================================
; Transmit the ASCII codes for a new line: carriage-return and line-feed.
; Registers modified: BSR
; Subroutines called: Txbyte
; Error code: none
Txnl
	movlb	0           ; 0 --> BSR, select bank 0
	movlw	CR          ; Carriage-Return --> W
	call	Txbyte      ; send W to host
	movlw	LF          ; Line-Feed --> W
	call    Txbyte      ; send W to host
	movlb	0           ; 0 --> BSR, select bank 0
	return	            ; bank 0
;============================================================================
; Transmit the byte in W as eight ASCII-binary characters.
; Registers modified: BSR, asc_lsn, bit_cnt, tmp_1
; Subroutines called: Txbyte
; Error code: none
Txwasbin
	movlb	0            ; 0 --> BSR, select bank 0
	movwf	tmp_1        ; W --> tmp_1
	movlw	D'8'         ; 8 --> W
	movwf	bit_cnt      ; W --> bit_counter
	movlw	0x30         ; ascii zero --> W
	movwf	asc_lsn      ; W --> asc_lsn
tby_20
	clrw	             ; 0 --> W
	rlf	tmp_1,f      ; rotate msb into Carry
	addwfc	asc_lsn,w    ; (W + asc_lsn + Carry) --> W
	call    Txbyte       ; send W to host
	decfsz  bit_cnt,f    ; dec bit_cnt
	bra	tby_20       ; branch back if bit_cnt > 0
	movlb	0           ; 0 --> BSR, select bank 0
	return	            ; bank 0
;============================================================================
; Transmit the Binary Angle Measure (BAM) in W as ASCII-decimal degrees.
; BAM ms-bit, binary weight of 128, has angular weight of 180 degrees.
; Conversion factor = 180/128 = 90/64 = 1.40625 degrees times BAM.
; Registers modified: BSR, ACa_n, ACb_n
; Subroutines called: S_mult, Copy_c2b, T_div, Txbasdec
; Error code: none
Txwasdeg
	movlb	0           ; 0 --> BSR, select bank 0
	movwf	ACa_0       ; move BAM to ACa_0
	movlw	D'90'       ; 90 --> W
	movwf	ACb_0       ;        W --> ACb_0
	call	S_mult      ; (BAM * 90) --> ACc
	call	Copy_c2b    ; (BAM * 90) --> ACb
	clrf	ACa_2       ; clear upper 
	clrf	ACa_1       ;  two bytes of ACa
	movlw	D'64'       ; 90/64 = 1.40625
	movwf	ACa_0       ; 64 --> ACa
	call	T_div       ; ACb/ACa = (BAM * 90)/64 --> ACb
	call	Txbasdec    ; send ACb as ASCII-decimal string
	movlb	0           ; 0 --> BSR, select bank 0
	return	            ; bank 0
;============================================================================
; Transmit the unsigned value in W or ACb as ASCII-decimal characters.
; Leading zeros are suppressed.
; Registers modified: BSR, FSR0, ACb_n, bcd_n, c_hold, dig_cnt, tmp_1
; Subroutines called: T_b2bcd, Txbyte
; Error code: none
Txwasdec	; send the value in W as ascii decimal characters
	movlb	0           ; 0 --> BSR, select bank 0
	movwf	ACb_0       ; W --> ACb_0
	clrf	ACb_1       ; clear upper bytes of
	clrf	ACb_2       ;  24-bit accumulator b
Txbasdec	; send the value in ACb as ascii decimal characters
	movlb	0           ; 0 --> BSR, select bank 0
	call	T_b2bcd     ; convert ACb to BCD
	movlw	bcd_7       ; Set pointer for
	movwf	FSR0L       ;  most significant bcd digit
	clrf	FSR0H
	movlw	D'8'        ; eight bcd digits
	movwf	dig_cnt     ; 8 --> digit counter
	movlw	0x0F        ; 0x0F --> W
	movwf	c_hold      ;          W --> leading zero flag
tad_10
	moviw	FSR0++      ; next bcd digit --> W
	movwf	tmp_1       ;  save it in tmp_1
	btfss	STATUS,Z    ; is it zero?
	bra	tad_20      ; - branch if <> 0
	movf	c_hold,f    ; test leading zero flag
	btfsc	STATUS,Z    ; is it zero?
	bra	tad_20      ; - branch of 0
	decf	dig_cnt,w   ; dig_cnt - 1 --> W
	btfss	STATUS,Z    ; is it last digit?
	bra	tad_30
tad_20
	clrf	c_hold      ; clear leading zero flag
	movf	tmp_1,w     ; next bcd digit --> W
	iorlw	A'0'        ; make it an ASCII character
	call	Txbyte      ; send it
tad_30
	decfsz	dig_cnt,f   ; decrement digit counter
	bra	tad_10      ; branch if not done
	movlb	0           ; 0 --> BSR, select bank 0
	return	            ; bank 0
;============================================================================
; Transmit the byte in W as two ascii-hex characters.  This subroutine falls
; through to txbyte so it doesn't have its own return instruction. 
; Registers modified: asc_msn, asc_lsn
; Subroutines called: Hex2asc, Txbyte
; Error code: none
Txwashex
	call	Hex2asc        ; Convert byte in W to two ascii characters
	movf	asc_msn,w      ; Get the character for the upper nibble
	call	Txbyte         ;  then send it to the host
	movf	asc_lsn,w      ; Get the character for the lower nibble
	call	Txbyte
	movlb	0             ; 0 --> BSR, select bank 0
	return	              ; bank 0
;============================================================================
; Transmit the byte in W to the host using the communications mode set
; by jumpers and stored in (mode). Return is from the subroutine.
; Registers modified: tx_buf
; Error code: none
Txbyte
	movlb	0            ; 0 --> BSR, select bank 0
	movwf	tx_buf       ; W --> tx_buf, store the byte to send
	btfss	mode,hcm     ; test communications mode
	goto	SPI_tx      ; 0 = SPI mode,  RB7 jumpered to Gnd
	goto	UART_tx     ; 1 = UART mode, no jumpers
;============================================================================
; Wait for the UART transmit buffer to empty then send the byte in (tx_buf).
; Registers modified: BSR, TXREG
; Subroutines called: none
; Error code: none
UART_tx
	banksel	PIR1          ; select bank 0
	btfss	PIR1,TXIF     ; wait for TX buffer to empty, TXIF=1
	goto	$-1           ; loop if TXIF = 0
	movf	tx_buf,w      ; tx_buf --> W
	banksel	TXREG         ; select bank 3
	movwf	TXREG         ; W --> TXREG, start transmission
	movlb	0             ; 0 --> BSR, select bank 0
	return	              ; bank 0
;============================================================================
; Wait for the SPI buffer to empty then send the byte in (tx_buf).
; Registers modified: BSR, 
; Subroutines called: none
; Error code: none
SPI_tx
	movlb	4             ; select bank 4 for SSP registers
	movf	SSP1BUF,w     ; read SSP1BUF to clear BF flag
	bcf	SSP1CON1,WCOL ; clear write collision flag
; SPI Slave Select (RB5) will be active if previous transmission not done.
	movlb	0             ; select bank 0
	btfss	PORTB,5       ;  loop while SPI Slave Select (RB5)
	bra	$-1           ;   still low (active)
; previous transmission done, load SSP1BUF with next byte to transmit
	movf	tx_buf,w      ; tx_buf --> W
	movlb	4             ; select bank 4 for SSP registers
	movwf	SSP1BUF       ;  data from W to SPI buffer
; signal host new byte is ready, set BRDY (RB0)
	banksel	LATB          ; select bank 2 for port output latches
	bsf	LATB,0        ;  set BRDY (RB0)
; wait until host starts transmission, Slave Select low
	movlb	0             ; select bank 0
	btfsc	PORTB,5       ;  loop while SPI Slave Select (RB5)
	bra	$-1           ;   still high (active)
; host is shifting new byte, clear BRDY (RB0)
	banksel	LATB          ; select bank 2 for port output latches
	bcf	LATB,0        ;  clear BRDY (RB0)
	movlb	0             ; 0 --> BSR, select bank 0
	return	              ; bank 0
;============================================================================
; Send message to the host terminal.
; Call with the Linear address of the message in FSR0. The message should
; be stored as a series of RETLW K instructions where K is the byte to be
; sent to the host. Transmission will continue until K=0.
; Registers modified: FSR0
; Subroutines called: Txbyte
; Error code: none
Msgout	moviw	FSR0++       ; read byte pointed to by FSR0 then increment
	btfsc	STATUS,Z     ; is the byte = 00 ?  (Z = 1)
	return	             ;   return if byte = 00
	call	Txbyte       ;   otherwise send byte to host
	bra	Msgout       ; get next byte
;============================================================================
; Acquire 127 10-bit samples from all three channels.
; Delay value of 15d = 0Fh produces a sampling frequency of 20 kHz.
; Registers modified: byt_cnt, FSR0
; Subroutines called: Grab10x3, Delay
; Error code: none
Acq127x10x3
	movlb	0x00         ; select bank 0
	movlw	low c3dat    ; low byte of buffer address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high c3dat   ; high byte of buffer address
	movwf	FSR0H        ;  --> FSR0H
	movlw	D'127'       ; collect 127 samples
	movwf	byt_cnt      ; in bank 0
Acq_10x3
	call	Grab10x3     ; 10-bit sample for all 3 channels
	movlw	D'15'        ; delay value for 20 kHz
	call	Delay        ; delay between samples 
	decfsz	byt_cnt,f    ; decrement counter
	bra	Acq_10x3     ; 2 cycle
	movlb	0             ; 0 --> BSR, select bank 0
	return	              ; bank 0
;============================================================================
; Acquire one 10-bit value from all three ADCs simultaneously.
; Store interleaved data in five bytes of buffer pointed to by FSR0.
; The entire acquisition requires 11 ADC clock pulses.
; On return FSR0 points to the next byte in the buffer, ADC clock = 0,
; and ADC chip select = 1. Data is stored in the following format:
;            b7  b6  b5  b4  b3  b2  b1  b0
; byte 0     0  d93 d92 d91  0  d83 d82 d81
; byte 1     0  d73 d72 d71  0  d63 d62 d61
; byte 2     0  d53 d52 d51  0  d43 d42 d41
; byte 3     0  d33 d32 d31  0  d23 d22 d21
; byte 4     0  d13 d12 d11  0  d03 d02 d01
; Registers modified: adc_1, FSR0
; Subroutines called: none
; Error code: none
Grab10x3
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,6       ; 0 --> ADC chip select
	nop
	nop
; clock pulse 1, charge sample & hold capacitor
	bsf	LATA,7       ; 1 --> ADC clock
	nop
	nop
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 2, charge sample & hold capacitor
	bsf	LATA,7       ; 1 --> ADC clock
	nop
	nop
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 3, ignore null data bit
	bsf	LATA,7       ; 1 --> ADC clock
	nop
	nop
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 4, d9 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 5, d8 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 6, d7 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 7, d6 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 8, d5 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 9, d4 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 10, d3 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 11, d2 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 12, d1 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
; clock pulse 13, d0 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
;
	bsf	LATA,6       ; 1 --> ADC chip select
	movlb	0            ; 0 --> BSR, select bank 0
	return	             ; bank 0
;============================================================================
; Acquire 127 8-BIT samples from all three channels.
; Delay value of 17d = 11h produces a sampling frequency of 20 kHz.
; Registers modified: byt_cnt, FSR0
; Subroutines called: Grab8x3, Delay
; Error code: none
Acq127x8x3
	movlb	0x00         ; select bank 0
	movlw	low abuff    ; low byte of buffer address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high abuff   ; high byte of buffer address
	movwf	FSR0H        ;  --> FSR0H
	movlw	D'127'       ; collect 127 samples
	movwf	byt_cnt      ; in bank 0
Acq_8x3
	call	Grab8x3      ; 8-bit sample for all 3 channels
	movlw	D'17'        ; delay value for 20 kHz
	call	Delay        ; delay between samples
	decfsz	byt_cnt,f    ; decrement counter
	bra	Acq_8x3      ; 2 cycle
	movlb	0            ; 0 --> BSR, select bank 0
	return	             ; bank 0
;============================================================================
; Acquire one byte from all three ADCs simultaneously.
; Store interleaved data in four bytes of buffer pointed to by FSR0.
; The entire acquisition requires 11 ADC clock pulses.
; On return FSR0 points to the next byte in the buffer, ADC clock = 0,
; and ADC chip select = 1. Data is stored in the following format:
;            b7  b6  b5  b4  b3  b2  b1  b0
; byte 0     0  d73 d72 d71  0  d63 d62 d61
; byte 1     0  d53 d52 d51  0  d43 d42 d41
; byte 2     0  d33 d32 d31  0  d23 d22 d21
; byte 3     0  d13 d12 d11  0  d03 d02 d01
; Registers modified: adc_1, FSR0
; Subroutines called: none
; Error code: none
Grab8x3
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,6       ; 0 --> ADC chip select
	nop
; clock pulse 1, charge sample & hold capacitor
	bsf	LATA,7       ; 1 --> ADC clock
	nop
	nop
	nop
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 2, charge sample & hold capacitor
	bsf	LATA,7       ; 1 --> ADC clock
	nop
	nop
	nop
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 3, ignore null data bit
	bsf	LATA,7       ; 1 --> ADC clock
	nop
	nop
	nop
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 4, d7 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 5, d6 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 6, d5 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 7, d4 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 8, d3 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 9, d2 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 10, d1 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	movwf	adc_1        ; 00000ddd --> adc_1
	swapf	adc_1,f      ; 0ddd0000 --> adc_1
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
	nop
	nop
	nop
; clock pulse 11, d0 bit from each ADC
	bsf	LATA,7       ; 1 --> ADC clock
	movlb	0x00         ; select bank 0 for PORTA
	movf	PORTA,w      ; xxxxxddd --> W
	andlw	0x07         ; 00000ddd --> W
	iorwf	adc_1,w      ; 0ddd0ddd --> W
	movwi	FSR0++       ; W to buffer then inc FSR0
	movlb	0x02         ; select bank 2 for LATA
	bcf	LATA,7       ; 0 --> ADC clock
;
	bsf	LATA,6       ; 1 --> ADC chip select
	movlb	0            ; 0 --> BSR, select bank 0
	return	             ; bank 0
;============================================================================
; Separate the 8-bit data in the acquisition buffer into three ADC signals
; in c1_dat, c2_dat and c3_dat; each contains 127 bytes of data.
; Registers modified: bit_cnt, byt_cnt, off_set, tmp_1, adc_1, adc_2,
;                     adc_3, c1dat, c2dat, c3dat, FSR0, FSR1
; Subroutines called: none
; Error code: none
Sort8x3
	movlb	0            ; 0 --> BSR, select bank 0
	clrf	off_set      ; offset pointer in c1/c2/c3dat arrays
	movlw	D'127'       ; process 127 samples (bytes) per channel
	movwf	byt_cnt      ; load byte downcounter 
	movlw	low abuff    ; low byte of acquisition buffer address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high abuff   ; high byte of acquisition buffer address
	movwf	FSR0H        ;  --> FSR0H
Sort10
	movlw	4          ; four bytes from abuff 
	movwf	bit_cnt      ; becomes three bytes  of adc data
Sort11
	moviw	FSR0++     ; two bits of interleaved adc data
	movwf	tmp_1        ; store it
	lslf	tmp_1,f      ; discard b7 bit
	lslf	tmp_1,f      ; b6, channel 3 bit --> C
	rlf	adc_3,f      ; rotate C into ls-bit of adc_3
	lslf	tmp_1,f      ; b5, channel 2 bit --> C
	rlf	adc_2,f      ; rotate C into ls-bit of adc_2
	lslf	tmp_1,f      ; b4, channel 1 bit --> C
	rlf	adc_1,f      ; rotate C into ls-bit of adc_1
	lslf	tmp_1,f      ; discard b3 bit
	lslf	tmp_1,f      ; b2, channel 3 bit --> C
	rlf	adc_3,f      ; rotate C into ls-bit of adc_3
	lslf	tmp_1,f      ; b1, channel 2 bit --> C
	rlf	adc_2,f      ; rotate C into ls-bit of adc_2
	lslf	tmp_1,f      ; b0, channel 1 bit --> C
	rlf	adc_1,f      ; rotate C into ls-bit of adc_1
	decfsz	bit_cnt,f    ; decrement bit counter
	goto	Sort11       ; need to get all 8 bits
;
	movlw	low c1dat    ; low byte of channel 1 adc data address
	movwf	FSR1L        ;  --> FSR1L
	movlw	high c1dat   ; high byte of channel 1 adc data address
	movwf	FSR1H        ;  --> FSR1H
	movf	off_set,w    ; offset pointer into adc data arrays
	addwf	FSR1L,f      ; add offset to FSR1L
	clrw	             ; clear W, Carry is unaffected
	addwfc	FSR1H,f      ; add any Carry to FSR1H 	
	movf	adc_1,w      ; adc_1 --> W
	movwf	INDF1        ; W --> (FSR1)
;
	movlw	low c2dat    ; low byte of channel 2 adc data address
	movwf	FSR1L        ;  --> FSR1L
	movlw	high c2dat   ; high byte of channel 2 adc data address
	movwf	FSR1H        ;  --> FSR1H
	movf	off_set,w    ; offset pointer into adc data arrays
	addwf	FSR1L,f      ; add offset to FSR1L
	clrw	             ; clear W, Carry is unaffected
	addwfc	FSR1H,f      ; add any Carry to FSR1H 	
	movf	adc_2,w      ; adc_2 --> W
	movwf	INDF1        ; W --> (FSR1)
;
	movlw	low c3dat    ; low byte of channel 3 adc data address
	movwf	FSR1L        ;  --> FSR1L
	movlw	high c3dat   ; high byte of channel 3 adc data address
	movwf	FSR1H        ;  --> FSR1H
	movf	off_set,w    ; offset pointer into adc data arrays
	addwf	FSR1L,f      ; add offset to FSR1L
	clrw	             ; clear W, Carry is unaffected
	addwfc	FSR1H,f      ; add any Carry to FSR1H 	
	movf	adc_3,w      ; adc_3 --> W
	movwf	INDF1        ; W --> (FSR1)
;
	incf	off_set,f    ; increment the offset into adc data arrays 
	decfsz	byt_cnt,f    ; decrement the byte count
	goto	Sort10       ; go if all 127 bytes not done
	return               ; bank 0
;============================================================================
; For each channel compute: 
; 1) the range of values (max-min), 
; 2) the average value; this equates to the analog ground voltage
;	plus the op-amp offset voltages.
; Determine whether the range for all three channels > min_rng.
; Registers modified: bit_cnt, byt_cnt, tmp_1, tmp_2, ACa, ACb, FSR0
;                     gnd_1, rng_1, gnd_2, rng_2, gnd_3, rng_3, freg
; Subroutines called: T_add, Chn_scan
; Error code: none
Chn_stat
	movlb	0            ; 0 --> BSR, select bank 0
	movlw	low c1dat    ; low byte of channel 1 adc data address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high c1dat   ; high byte of channel 1 adc data address
	movwf	FSR0H        ;  --> FSR0H
	call	Chn_scan
	movf	tmp_1,w      ; analog ground value --> W
	movwf	gnd_1
	movf	tmp_2,w      ; range --> W
	movwf   rng_1
;
	movlw	low c2dat    ; low byte of channel 2 adc data address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high c2dat   ; high byte of channel 2 adc data address
	movwf	FSR0H        ;  --> FSR0H
	call	Chn_scan
	movf	tmp_1,w      ; analog ground value --> W
	movwf	gnd_2
	movf	tmp_2,w      ; range --> W
	movwf   rng_2
;
	movlw	low c3dat    ; low byte of channel 3 adc data address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high c3dat   ; high byte of channel 3 adc data address
	movwf	FSR0H        ;  --> FSR0H
	call	Chn_scan
	movf	tmp_1,w      ; analog ground value --> W
	movwf	gnd_3
	movf	tmp_2,w      ; range --> W
	movwf   rng_3
;
; all ranges must be > min_rng for sufficient signal to process
	bsf	freg,rok     ; set ranges ok flag
 	movf	rng_1,w     ; channel 1 range --> W
	subwf	min_rng,w    ; min_rng - rng_1 --> W
	btfsc	STATUS,C     ; C=0 if rng_1 > min_rng
	bcf	freg,rok     ; C=1, rng_1 <= min_rng, clear flag
 	movf	rng_2,w     ; channel 2 range --> W
	subwf	min_rng,w    ; min_rng - rng_2 --> W
	btfsc	STATUS,C     ; C=0 if rng_2 > min_rng
	bcf	freg,rok     ; C=1, rng_2 <= min_rng, clear flag
 	movf	rng_3,w     ; channel 3 range --> W
	subwf	min_rng,w    ; min_rng - rng_3 --> W
	btfsc	STATUS,C     ; C=0 if rng_3 > min_rng
	bcf	freg,rok     ; C=1, rng_3 <= min_rng, clear flag
	return              ; bank 0
;
; Scan the 127 values in the channel buffer pointed to by FSR0.
; Find the min and max and compute the range of values in tmp_2.
; Sum all 127 values, divide by 128, add one, return result tmp_1.
Chn_scan
	clrf    ACa_2        ; clear ACa top two bytes
	clrf    ACa_1
	clrf    ACb_2        ; clear ACb to hold sum
	clrf    ACb_1
	clrf    ACb_0
	movlw	D'127'       ; process 127 samples (bytes) per channel
	movwf	tmp_1        ; preload min 
	movwf	tmp_2        ; preload max
	movwf	byt_cnt      ; load byte downcounter 
;
Chn_10
	movf	tmp_1,w      ; get the min
	subwf	INDF0,w      ; (FSR0)-min --> W
	btfsc	STATUS,Z     ; does (FSR0) = min ?
	bra	Chn_20       ; - branch if (FSR0) = min
	btfsc	STATUS,C     ; is (FSR0) > or < min ?
	bra	Chn_20       ; - branch if (FSR0) > min, C=1
	movf	INDF0,w      ; - otherwise (FSR0) < min, C=0
	movwf	tmp_1        ;   new value to min
Chn_20
	movf	tmp_2,w      ; get the max
	subwf	INDF0,w      ; (FSR0)-max --> W
	btfsc	STATUS,Z     ; does (FSR0) = max ?
	bra	Chn_30       ; - branch if (FSR0) = max
	btfss	STATUS,C     ; is (FSR0) > or < max ?
	bra	Chn_30       ; - branch if (FSR0) < max, C=0
	movf	INDF0,w      ; - otherwise (FSR0) > min, C=1
	movwf	tmp_2        ;   new value to max
Chn_30
	moviw	FSR0++       ; read byte from channel buffer
	movwf	ACa_0        ;  move it to ACa
	call	T_add        ;   ACa + ACb --> ACb
	decfsz	byt_cnt,f    ;    decrement byte counter
	bra	Chn_10       ;     loop for all 127 bytes
;
	movf	tmp_1,w      ; min --> W
	subwf	tmp_2,f      ; tmp_2-tmp_1 (range) --> tmp_2
	movlw	D'7'         ; Divide Acb by 128, which is 7 shifts right
	movwf	bit_cnt      ; shift counter
Chn_70
	lsrf	ACb_2,f      ; 0xxxxxxx --> ACb_2, lsb --> Carry
	rrf	ACb_1,f      ; Cxxxxxxx --> ACb_1, lsb --> Carry
	rrf	ACb_0,f      ; Cxxxxxxx --> ACb_0
	decfsz	bit_cnt,f    ; done with all 7 shifts?
	bra	Chn_70
	incf	ACb_0,w      ; compensate for 127(x)/128
	movwf	tmp_1        ; average --> tmp_1
	return               ;
;============================================================================
; Compute a bearing based on the amplitude (rng) of the three channels.
; The three amplitudes are considered vectors 120 degrees apart.
; Registers modified: BSR, all 3-byte accumulators
; Subroutines called: T_add, T_sub, T_mult
; Error code: none
Chn_amp
	movlb	0            ; 0 --> BSR, select bank 0
; X component (Rx) = 0.5*rng_1 + 0.5*rng_2 - rng_3
	clrf	ACa_2       ; clear accumulator A
	clrf	ACa_1       ;  upper two bytes
	movf	rng_1,w     ;   rng_1 --> W
	movwf	ACa_0       ;    ACa now equals rng_1
	clrf	ACb_2       ; clear accumulator B
	clrf	ACb_1       ;  upper two bytes
	movf	rng_2,w     ;   rng_2 --> W
	movwf	ACb_0       ;    ACb now equals rng_2
	call	T_add       ; ACb + ACa --> ACb, ACa unchanged
	lsrf	ACb_1,f     ;  ACb = rng_1 + rng_2
	rrf	ACb_0,f     ;   ACb now = 0.5*(rng_1 + rng_2)
	movf	rng_3,w     ; rng_3 --> W
	movwf	ACa_0       ;  ACa now equals rng_3
	call	T_sub       ; ACb - ACa --> ACb
	movf	ACb_2,w     ; move Rx, which equals
	movwf	ACx_2       ;  0.5*(rng_1 + rng_2) - rng_3
	movf	ACb_1,w     ;   from ACb
	movwf	ACx_1       ;    to ACx
	movf	ACb_0,w     ;     for
	movwf	ACx_0       ;      future use
; Y component (Ry) = 0.866*rng_1 - 0.866*rng_2 
;  The ratio used for arctan lookup is 16*(Ry/Rx) = (16*Ry)/Rx
;  16*Ry = 16*(0.866)*rng_1 - 16*(0.866)*rng_2 ~ 14*rng_1 - 14*rng_2
	movf	rng_1,w     ; rng_1 --> W
	movwf	ACa_0       ;           W --> ACa_0
	movlw	D'14'       ; 14 --> W
	movwf	ACb_0       ;        W --> ACb_0
	call	S_mult      ; ACb_0 * ACa_0 --> ACc = 14*rng_1
	movf	ACc_2,w     ; move
	movwf	ACy_2       ;  14*rng_1
	movf	ACc_1,w     ;   from ACc
	movwf	ACy_1       ;    to ACy
	movf	ACc_0,w     ;     for
	movwf	ACy_0       ;      future use
	movf	rng_2,w     ; rng_2 --> W
	movwf	ACa_0       ;           W --> ACa_0
	movlw	D'14'       ; 14 --> W
	movwf	ACb_0       ;        W --> ACb_0
	call	S_mult      ; ACb_0 * ACa_0 --> ACc = 14*rng_2
	call	Copy_c2a    ; ACc --> ACa = 14*rng_2
	call	Copy_y2b    ; ACy --> ACb = 14*rng_1
	call	T_sub       ; ACb - ACa --> ACb = 16*Ry
	movf	ACb_2,w     ; move
	movwf	ACy_2       ;  16*Ry
	movf	ACb_1,w     ;   from ACb
	movwf	ACy_1       ;    to ACy
	movf	ACb_0,w     ;     for 
	movwf	ACy_0       ;      future use
; Find the quadrant for the arctan
;  Make both Rx and Rx positive before unsigned divide 
	btfss	ACx_2,7     ; test X sign bit
	bra	X_pos       ; - X positive
	bra	X_neg       ; - Y negative
X_pos
	btfss	ACy_2,7     ; test Y sign bit
	bra	arc_q1      ; - X pos, Y pos
	bra	arc_q4      ; - X pos, Y neg
X_neg
	btfss	ACy_2,7     ; test Y sign bit
	bra	arc_q2      ; - X neg, Y pos
	bra	arc_q3      ; - X neg, Y neg
arc_q1	; quadrant 1 - X pos, Y pos
	clrw	             ; 00 --> b3,b2
	bra	q_set
arc_q2	; quadrant 2 - X neg, Y pos
	call	Neg_ACx
	movlw	B'00000100'  ; 01 --> b3,b2
	bra	q_set
arc_q3	; quadrant 3 - X neg, Y neg
	call	Neg_ACx
	call	Neg_ACy
	movlw	B'00001000'  ; 10 --> b3,b2
	bra	q_set
arc_q4	; quadrant 4 - X pos, Y neg
	call	Neg_ACy
	movlw	B'00001100'  ; 11 --> b3,b2
q_set	; store quadrant bits in index01
	movwf	indx1       ; quadrant bits --> indx1
; Test for 16*Ry > Rx before calling division
	call	Copy_y2b    ; ACy --> ACb = 16*Ry
	call	Copy_x2a    ; ACx --> ACa = Rx
	call	T_sub       ; ACb - ACa --> ACb = 16*Ry - Rx 
	btfsc	ACb_2,7     ; test sign bit
	goto	sec_set     ; - go if Rx > 16*Ry
; else 16*Ry >= Rx, compute the ratio (16*Ry)/Rx
	call	Copy_y2b    ; ACy --> ACb = 16*Ry
	call	Copy_x2a    ; ACx --> ACa = Rx
	call	T_div       ; ACb/ACa --> ACb
; Find the arctan sector in the quadrant already found
	clrf	ACa_2      ; clear ACa_2
	clrf	ACa_1       ; clear ACa_1
	movlw	D'5'        ; 5 --> W
	movwf	ACa_0       ;       W --> ACa_0
	call	T_sub       ; ACb - ACa --> ACb
	btfsc	ACb_2,7     ; test sign bit
	goto	sec_set     ; - go if (16*Ry/Rx) < 5
	incf	indx1,f    ; indx1+1 --> indx1
	clrf	ACa_2       ; clear ACa_2
	clrf	ACa_1       ; clear ACa_1
	movlw	D'12'       ; 12 --> W
	movwf	ACa_0       ;        W --> ACa_0
	call	T_sub       ; ACb - ACa --> ACb
	btfsc	ACb_2,7     ; test sign bit
	goto	sec_set     ; - go if (16*Ry/Rx) < 17
	incf	indx1,f    ; indx1+1 --> indx1
	clrf	ACa_2       ; clear ACa_2
	clrf	ACa_1       ; clear ACa_1
	movlw	D'43'       ; 43 --> W
	movwf	ACa_0       ;        W --> ACa_0
	call	T_sub       ; ACb - ACa --> ACb
	btfsc	ACb_2,7     ; test sign bit
	bra	sec_set     ; - go if (16*Ry/Rx) < 60
	incf	indx1,f    ; 16*Ry/Rx) >= 60, indx1+1 --> indx1
sec_set
	movlw	low Tbl_amp   ; low byte of table data address
	movwf	FSR1L         ;  --> FSR1L
	movlw	high Tbl_amp  ; high byte of table data address
	movwf	FSR1H         ;  --> FSR1H
	movf	indx1,w       ; index into table --> W
	addwf	FSR1L,f       ; add index to FSR1L
	clrw	              ; clear W, Carry is unaffected
	addwfc	FSR1H,f       ; add any Carry to FSR1H 	
	movf	INDF1,w       ; (FSR1) --> W
	movwf	brng_a        ; store center angle of sector
	movlb	0            ; 0 --> BSR, select bank 0
	return               ; bank 0
;============================================================================
; Convert each channel from unsigned to signed 8-bit data.
; Subtract the average for channel N (gnd_N) from each data point.
; The resultant 2-complement signed value is limited to the range
; -127 (81h) to +127 (7Fh). 
; Registers modified: BSR, FSR0, byt_cnt, tmp_1, adc_2, channel buffers
; Subroutines called: Sub_gnd
; Error code: none
Chn_sign
	movlb	0            ; 0 --> BSR, select bank 0
	movlw	low c1dat    ; low byte of channel 1 adc data address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high c1dat   ; high byte of channel 1 adc data address
	movwf	FSR0H        ;  --> FSR0H
	movf	gnd_1,w      ; analog ground value --> W
	movwf	tmp_1        ;  W --> tmp_1
	call	Sub_gnd      ; convert channel 1 to signed 8-bit data
;
	movlw	low c2dat    ; low byte of channel 2 adc data address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high c2dat   ; high byte of channel 2 adc data address
	movwf	FSR0H        ;  --> FSR0H
	movf	gnd_2,w      ; analog ground value --> W
	movwf	tmp_1        ;  W --> tmp_1
	call	Sub_gnd      ; convert channel 2 to signed 8-bit data
;
	movlw	low c3dat    ; low byte of channel 3 adc data address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high c3dat   ; high byte of channel 3 adc data address
	movwf	FSR0H        ;  --> FSR0H
	movf	gnd_3,w      ; analog ground value --> W
	movwf	tmp_1        ;  W --> tmp_1
	call	Sub_gnd      ; convert channel 3 to signed 8-bit data
	return               ; bank 0
;
; Compute the signed 8-bit values for the 127 values in the channel 
; buffer pointed to by FSR0.
Sub_gnd
	movlb	0           ; 0 --> BSR, select bank 0
	movlw	D'127'      ; process 127 samples (bytes) per channel
	movwf	byt_cnt     ; load byte downcounter 
Sg_10
	movf	tmp_1,w     ; get the ground reference value
	subwf	INDF0,w     ;  data - gnd --> W
	movwf	adc_2       ;   W --> adc_2
	btfss	STATUS,C
	bra	r_neg
	bra	r_pos
r_neg
	xorlw	0x80        ; if result = -128
	btfsc	STATUS,Z    ;  then limit it
	bra	r_n20       ;   to -127
	btfsc	adc_2,7     ; if result is negative b7 should be 1	
	bra	r_ok        ;  branch if b7 shows negative
r_n20
	movlw	0x81        ;   if rollunder happened then
	movwf	adc_2       ;    cap result at -127
	bra	r_ok
r_pos
	btfss	adc_2,7     ; if result is positive b7 should be 0
	bra	r_ok        ;  branch if b7 shows positive
	movlw	0x7F        ;   if rollover happened then
	movwf	adc_2       ;    cap result at +127
r_ok
  	movf	adc_2,w     ; result --> W, write W to channel buffer
	movwi	FSR0++      ;  then increment FSR0
	decfsz	byt_cnt,f   ; decrement byte counter
	bra	Sg_10       ;  loop for all 127 bytes
	return
;============================================================================
; Compute the cross correlation of channel pairs. Each channel is 127 points,
; indices from 0 to 126, center at index 63. The cross-correlation go from 
; lag=-6 to lag=+6 (13 total), 115 product pairs for each lag value.
; Registers modified: BSR, FSR0, FSR1, ACa, ACb, ACc, ACd, ACz
;                     xc_lc, xc_sc, lag_12, lag_23, lag_31
; Subroutines called: Pair1
; Error code: none
c1off	equ	c1dat+D'6'   ; starting index for c1 as baseline
c2off	equ	c2dat+D'6'   ; starting index for c2 as baseline
c3off	equ	c3dat+D'6'   ; starting index for c3 as baseline
X_corr
	movlb	0            ; 0 --> BSR, select bank 0
; set up for cross-correlation of channels 1 & 2
	clrf	ACz_2         ; clear the maximum cross-correlation
	clrf	ACz_1         ;  cross-correlation
	clrf	ACz_0         ;   sum for baseline 1-2
	clrf	off_set       ; 0 --> offset, starting point in channel 2
	movlw	D'13'         ; 13 lags, -6 to +6
	movwf	xc_lc         ; 13 --> xc_lc, lag counter
xc_12
	movlw	low c1off     ; low byte of offset address into  
	movwf	FSR0L         ;  channel 1 adc data --> FSR0L
	movlw	high c1off    ; high byte of  offset address into
	movwf	FSR0H         ;  channel 1 adc data --> FSR0H
	movlw	low c2dat     ; low byte of channel 2 adc data 
	movwf	FSR1L         ;  address --> FSR1L
	movlw	high c2dat    ; high byte of channel 2 adc data 
	movwf	FSR1H         ;  address --> FSR1H
	movf	off_set,w     ; offset pointer into channel 2 data
	addwf	FSR1L,f       ;  add offset to FSR1L
	clrw	              ;   clear W, Carry is unaffected
	addwfc	FSR1H,f       ;    add any Carry to FSR1H 	
	call	Pair1       ; add xc_sc product pairs to form xc sum
	call	Copy_d2b      ; sum at this lag to ACb
	call	Copy_z2a      ; max lag sum for baseline 1-2
	call	T_sub         ; ACb - ACa --> ACb
	btfsc	ACb_2,7       ; test sign bit of (sum - max)
	bra	xc_12w        ; - branch if max > sum
	movf	xc_lc,w       ; lag count --> W
	sublw	D'7'          ; 7 - xc_lc --> W
	movwf	lag_12        ; update max lag index
	call	Copy_d2z      ; update max lag sum
xc_12w
	incf	off_set,f    ; increment offset into channel 2 data
	decfsz	xc_lc,f      ; decrement lag counter
	bra	xc_12        ;  branch back if > 0
;
; set up for cross-correlation of channels 2 & 3
	clrf	ACz_2         ; clear the maximum cross-correlation
	clrf	ACz_1         ;  cross-correlation
	clrf	ACz_0         ;   sum for baseline 2-3
	clrf	off_set       ; 0 --> offset, starting point in channel 3
	movlw	D'13'         ; 13 lags, -6 to +6
	movwf	xc_lc         ; 13 --> xc_lc, lag counter
xc_23
	movlw	low c2off     ; low byte of offset address into  
	movwf	FSR0L         ;  channel 2 adc data --> FSR0L
	movlw	high c2off    ; high byte of  offset address into
	movwf	FSR0H         ;  channel 2 adc data --> FSR0H
	movlw	low c3dat     ; low byte of channel 3 adc data 
	movwf	FSR1L         ;  address --> FSR1L
	movlw	high c3dat    ; high byte of channel 3 adc data 
	movwf	FSR1H         ;  address --> FSR1H
	movf	off_set,w     ; offset pointer into channel 3 data
	addwf	FSR1L,f       ;  add offset to FSR1L
	clrw	              ;   clear W, Carry is unaffected
	addwfc	FSR1H,f       ;    add any Carry to FSR1H 	
	call	Pair1       ; add xc_sc product pairs to form xc sum
	call	Copy_d2b      ; sum at this lag to ACb
	call	Copy_z2a      ; max lag sum for baseline 2-3
	call	T_sub         ; ACb - ACa --> ACb
	btfsc	ACb_2,7       ; test sign bit of (sum - max)
	bra	xc_23w        ; - branch if max > sum
	movf	xc_lc,w       ; lag count --> W
	sublw	D'7'          ; 7 - xc_lc --> W
	movwf	lag_23        ; update max lag index
	call	Copy_d2z      ; update max lag sum
xc_23w
	incf	off_set,f    ; increment offset into channel 3 data
	decfsz	xc_lc,f      ; decrement lag counter
	bra	xc_23        ;  branch back if > 0
;
; set up for cross-correlation of channels 3 & 1
	clrf	ACz_2         ; clear the maximum cross-correlation
	clrf	ACz_1         ;  cross-correlation
	clrf	ACz_0         ;   sum for baseline 3-1
	clrf	off_set       ; 0 --> offset, starting point in channel 1
	movlw	D'13'         ; 13 lags, -6 to +6
	movwf	xc_lc         ; 13 --> xc_lc, lag counter
xc_31
	movlw	low c3off     ; low byte of offset address into  
	movwf	FSR0L         ;  channel 3 adc data --> FSR0L
	movlw	high c3off    ; high byte of  offset address into
	movwf	FSR0H         ;  channel 3 adc data --> FSR0H
	movlw	low c1dat     ; low byte of channel 1 adc data 
	movwf	FSR1L         ;  address --> FSR1L
	movlw	high c1dat    ; high byte of channel 1 adc data 
	movwf	FSR1H         ;  address --> FSR1H
	movf	off_set,w     ; offset pointer into channel 1 data
	addwf	FSR1L,f       ;  add offset to FSR1L
	clrw	              ;   clear W, Carry is unaffected
	addwfc	FSR1H,f       ;    add any Carry to FSR1H 	
	call	Pair1       ; add xc_sc product pairs to form xc sum
	call	Copy_d2b      ; sum at this lag to ACb
	call	Copy_z2a      ; max lag sum for baseline 3-1
	call	T_sub         ; ACb - ACa --> ACb
	btfsc	ACb_2,7       ; test sign bit of (sum - max)
	bra	xc_31w        ; - branch if max > sum
	movf	xc_lc,w       ; lag count --> W
	sublw	D'7'          ; 7 - xc_lc --> W
	movwf	lag_31        ; update max lag index
	call	Copy_d2z      ; update max lag sum
xc_31w
	incf	off_set,f    ; increment offset into channel 1 data
	decfsz	xc_lc,f      ; decrement lag counter
	bra	xc_31        ;  branch back if > 0
;
; combine the 3 baseline lags to form index into bearing table
	clrf	indx2        ; clear index into Tbl_xcr
b12
	movf	lag_12,f      ; test lag_12 for zero
	btfsc	STATUS,Z      ; Z=1 if lag_12 = 0
	bra	b23           ; - branch if lag_12 = 0
	btfss	lag_12,7      ; test lag_12 sign bit
	bra	b12_pos       ; branch if lag_12 is positive
b12_neg
	bsf	indx2,4       ; set b12 negative bit
	bra	b23
b12_pos
	bsf	indx2,5       ; set b12 positive bit
b23
	movf	lag_23,f      ; test lag_23 for zero
	btfsc	STATUS,Z      ; Z=1 if lag_23 = 0
	bra	b31           ; - branch if lag_23 = 0
	btfss	lag_23,7      ; test lag_23 sign bit
	bra	b23_pos       ; branch if lag_23 is positive
b23_neg
	bsf	indx2,2       ; set b23 negative bit
	bra	b31
b23_pos
	bsf	indx2,3       ; set b23 positive bit
b31
	movf	lag_31,f      ; test lag_31 for zero
	btfsc	STATUS,Z      ; Z=1 if lag_31 = 0
	bra	b99           ; - branch if lag_31 = 0
	btfss	lag_31,7      ; test lag_31 sign bit
	bra	b31_pos       ; branch if lag_31 is positive
b31_neg
	bsf	indx2,0       ; set b31 negative bit
	bra	b99
b31_pos
	bsf	indx2,1       ; set b31 positive bit
b99
	movlw	low Tbl_xcr   ; low byte of table data address
	movwf	FSR1L         ;  --> FSR1L
	movlw	high Tbl_xcr  ; high byte of table data address
	movwf	FSR1H         ;  --> FSR1H
	movf	indx2,w       ; index into table --> W
	addwf	FSR1L,f       ; add index to FSR1L
	clrw	              ; clear W, Carry is unaffected
	addwfc	FSR1H,f       ; add any Carry to FSR1H 	
	movf	INDF1,w       ; (FSR1) --> W
	movwf	brng_x        ; store center angle of sector
	movlb	0            ; 0 --> BSR, select bank 0
	return               ; bank 0
;
; Compute the product of two signed bytes from the channel data buffers
; (in ACa_0 and ACb_0) then add the product to the cross-correlation sum
; (in ACd). Multiply and add happens xc_sc times with the pointers into 
; the data arrays incrementing each time. Preload FSR0 and FSR1.
Pair1
	clrf	ACd_2       ; clear ACd
	clrf	ACd_1       ;  which holds the
	clrf	ACd_0       ;   cross-correlation sum
	movlw	D'115'      ; 115 points in the cross-correlation
	movwf	xc_sc       ;  sum at each lag value
P1_00	; start of loop
	moviw	FSR0++       ; next byte --> W, then increment FSR0
	movwf	ACa_0        ;  W --> accumulator A ls-byte
	moviw	FSR1++       ; next byte --> W, then increment FSR1
	movwf	ACb_0        ;  W --> accumulator B ls-byte
	clrf	xc_nc        ; clear cross-correlation negative counter
P1_ACa	; test ACa_0
	btfss	ACa_0,7      ; test ACa_0 sign bit 
	bra	P1_ACb       ; - branch if >= 0, positive
	incf	xc_nc,f      ; else increment negative counter and
	comf	ACa_0,f      ;  make ACa_0 positive via 2s complement and
	incf	ACa_0,f      ;   add one
P1_ACb	; test ACb_0
	btfss	ACb_0,7      ; test ACb_0 sign bit
	bra	P1_mult      ; - branch if >= 0, positive
	incf	xc_nc,f      ; else increment negative counter and
	comf	ACb_0,f      ;  make ACb_0 positive via 2s complement and
	incf	ACb_0,f      ;   add one
P1_mult	; ACa_0 and ACb_0 are now both positive
	call	S_mult       ; ACa_0 * ACb_0 --> ACc, unsigned multiply
	call	Copy_c2a     ; product in ACc --> ACa
	call	Copy_d2b     ; xcorr sum in ACd --> ACb
	btfss	xc_nc,0      ; test negative counter lsb
	bra	P1_add       ; - product positive if -- (2) or ++ (0)
	bra	P1_sub       ; - product negative if +- (1) or -+ (1)
P1_add
	call	T_add        ; xcorr sum + product --> ACb
	bra	P1_90
P1_sub
	call	T_sub        ; xcorr sum - product  --> ACb
P1_90
	call	Copy_b2d     ; ACb --> xcorr sum in ACd
	decfsz	xc_sc,f      ; decrement xc sum counter
	bra	P1_00        ; branch back if counter > 0
	return
;
;****************************************************************************
;*****   Start program here, Power-On Reset (POR)
;****************************************************************************
;
MAIN                           ; (Beginning of program)
	clrf	STATUS         ; Do initialization, select bank 0 registers
	clrf	PCLATH         ; select program memory page 0
	banksel	OPTION_REG     ; select bank 1
	movlw	B'01111111'    ; Weak pullups enabled
	movwf	OPTION_REG
;
init_clock		       ; POR selects 500 kHz as Fosc
	banksel	OSCCON         ; select bank 1 
	;b7	SPLLEN         DISABLED  0
	;b6:3	IRCF           16 MHz    1111
	;b2	Unimplemented            0
	;b1:0	SCS            Internal  10
	movlw	B'01111010'    ; 16 MHz internal clock
	movwf	OSCCON
;
; POR disables all interrupts
;
; configure I/O ports
; PORT(A:B) = read, bank 0
; TRIS(A:B) = direction, bank 1
; LAT(A:B) = ouptut latch, bank 2
; ANSEL(A:B) = analog select, bank 3
; WPU(A:B) = weak pullup, bank 4
init_porta	; Port A is the same for all modes: UART, SPI
	banksel	TRISA         ; select bank 1
	movlw	B'00111111'   ;  TRISA; 1=In, 0=Out
	movwf	TRISA         ;   PORTA directions
	banksel	ANSELA        ; select bank 3
	movlw	B'00000000'   ;  no analog functions
	movwf	ANSELA        ;   on PORTA
	banksel	LATA          ; select bank 2
	movlw	B'01000000'   ;  ADC clock = 0,
	movwf	LATA          ;   ADC chip select not = 1
	; weak pullup for MCLR (RA5) on by default 
init_portb	
	banksel	TRISB         ; select bank 1
	movlw	B'11110011'   ;  TRISA; 1=In, 0=Out
	movwf	TRISB         ;   PORTB directions
	banksel	WPUB          ; select bank 4
	movwf	WPUB          ;  WPUB = TRISB, weak pullups on inputs
	banksel	ANSELB        ; select bank 3
	movlw	B'00000000'   ;  no analog functions 
	movwf	ANSELB        ;   on PORTB
	banksel	LATB          ; select bank 2
	movlw	B'00001100'   ;  RB3 = BUSY = 1,
	movwf	LATB          ;   RB2 = 1
;
init_mode
; RB7 determines communications mode
	banksel	PORTB         ; select bank 0
	movf	PORTB,w       ;  PORTB --> W
	andlw	B'10000000'   ;   70000000 --> W
	movwf	mode          ;   70000000 --> mode
; RA4:RA3 are unique setting for each mode
	banksel	PORTA         ; select bank 0
	movf	PORTA,w       ;  PORTA --> W
	andlw	B'00011000'   ;   00043000 --> W
	movwf	csetb         ;   00043000 --> csetb
	lsrf	csetb,f       ;   00004300 --> csetb
	lsrf	csetb,f       ;   00000430 --> csetb
	lsrf	csetb,f       ;   00000043 --> csetb
; initialize for selected communications mode
	btfss	mode,hcm      ; test communications mode
	goto	SPI_init      ; 0 = SPI mode,  RB7 jumpered to Gnd
	goto	UART_init     ; 1 = UART mode, no jumpers
;
SPI_init	; Mode 0 = SPI Mode
;Option jumpers RA<4:3> are stored in csetb<1:0>
;For SPI, RA4 is CKP and RA3 is CKE
	banksel	TRISB         ; select bank 1
	movlw	B'11110010'   ;  TRISA; 1=In, 0=Out
	movwf	TRISB         ;   PORTB directions
	banksel	WPUB          ; select bank 4
	movwf	WPUB          ;   WPUB = TRISB, pullups on inputs
	banksel	LATB          ; select bank 2
	movlw	B'00001100'   ;  RB3 = BUSY = 1, RB2 = 1,
	movwf	LATB          ;   RB0 = RDY = 0
	movlb	4             ; select bank 4 for all SSP registers
	clrf	SSP1STAT      ; b7(SMP)=0, b6(CKE)=0
	btfsc	csetb,0       ; csetb is in Common RAM
	bsf	SSP1STAT,CKE  ; set CKE=1
	movlb	0             ; select bank 0
	btfss	csetb,1       ; test CKP option input
	goto	ckp_0         ;  go if CKP=0
	goto	ckp_1         ;  go if CKP=1
ckp_0
	btfsc	PORTB,4       ; test SCK input from host
	goto	ckp_0         ;  loop while SCK=1
	movlw	B'00000100'   ; CKP=0, SPI Slave Mode, Slave Select
	bra	SPI_en
ckp_1
	btfss	PORTB,4       ; test SCK input from host
	goto	ckp_1         ;  loop while SCK=0
	movlw	B'00010100'   ; CKP=1, SPI Slave Mode, Slave Select
SPI_en
	movlb	4             ; select bank 4
	movwf	SSP1CON1      ; Set SPI options
	bsf	SSP1CON1,5    ; SSPEN=1, SPI mode enabled
	movf	SSP1BUF,w     ; read buffer to clear BF bit
	goto	Start
;
UART_init	; Mode 1 = UART Mode
;Option jumpers RA<4:3> are stored in csetb<1:0>
;For UART, RA<4:3> is the baud rate code
	movlw	low Tbl_baud  ; low byte of table data address
	movwf	FSR1L         ;  --> FSR1L
	movlw	high Tbl_baud ; high byte of table data address
	movwf	FSR1H         ;  --> FSR1H
	movf	csetb,w       ; index into table --> W
	addwf	FSR1L,f       ; add index to FSR1L
	clrw	              ; clear W, Carry is unaffected
	addwfc	FSR1H,f       ; add any Carry to FSR1H 	
	movf	INDF1,w       ; (FSR1) --> W
	movwf	tmp01         ; store baud rate value in tmp01
	banksel	SPBRGH        ; select bank 3 for all UART registers
	clrf	SPBRGH        ; load SPBRGH:SPBRGL
	movf	tmp01,w       ;  baud rate value --> W
	movwf	SPBRGL        ;   W --> SPBRGL
	movlw	B'00100111'   ; TX9=0, TXEN=1, SYNC=0, BRGH=1
	movwf	TXSTA         ;    --> tx status
	movlw	B'10010000'   ; SPEN=1, RX9=0, SREN=0, CREN=1
	movwf	RCSTA         ;    --> rx status
	movlw	B'01000000'   ; 8-bit baud rate generator
	movwf	BAUDCON       ;    --> baud rate control
	goto	Start
;
Start
	movlb	0            ; 0 --> BSR, select bank 0
	movlw	D'31'         ; 31 -->
	movwf	min_rng       ; default value for min_rng
	clrf	freg          ; clear flags
;
; ---------------------------------------------------------------------------
Host_01
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
	bcf	LATB,0       ;  clear BRDY line (RB0)
; Check for computer control mode or human readable menu
	btfsc	mode,ccm     ; Test computer control mode bit
	bra	Host_20      ; - go if ccm = 1
	call	Dlymax	     ; delay 
	movlw	low Menu1    ; low byte of message address
	movwf	FSR0L        ;  --> FSR0L
	movlw	high Menu1   ; high byte with b7 set 
	movwf	FSR0H        ;  --> FSR0H
	call	Msgout
	call	UART_clr     ; Clear UART errors and fifo
;
Host_20
	banksel	LATB         ; select bank 2
	bcf	LATB,3       ;  clear BUSY line (RB3)
; Get menu selection from host and decode.
	call	Rxbyte       ; Get a byte from the host in W, 
	movwf	tmp01        ;  save it in (tmp01),
	btfss	mode,ccm     ; Test computer control mode bit 
	call	Txbyte       ; - echo command if ccm = 0
; Numeric options
	movlw	A'@'          ; Reset ROBOEAR, ascii 0x40
	xorwf	tmp01,w        ; (tmp01) .XOR. '@'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was '@'
	reset	               ;  - if it was, reset
	movlw	ESC          ; End computer control mode, ascii 'Escape'
	xorwf	tmp01,w        ; (tmp01) .XOR. '0'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was '0'
	goto	Opt_ESC        ;  - if it was, jump

; Alpha options
        movlw	0xDF           ; AND with 0xDF clears bit-5 converting 
        andwf	tmp01,f        ;  character to upper case
	movlw	A'R'        ; Send raw data, ascii 'R'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'R'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'R'
	goto	Opt_R          ;  - if it was, jump
	movlw	A'B'        ; Send amplitude & XCORR bearings, ascii 'B'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'B'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'B'
	goto	Opt_B          ;  - if it was, jump
	movlw	A'E'        ; Acquire one set of eight-bit data, ascii 'E'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'E'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'E'
	goto	Opt_E          ;  - if it was, jump
	movlw	A'T'        ; Acquire one set of ten-bit data, ascii 'T'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'T'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'T'
	goto	Opt_T          ;  - if it was, jump
	movlw	A'P'        ; Acquire & process 1 set of 8-bit data, ascii 'P'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'P'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'P'
	goto	Opt_P          ;  - if it was, jump
	movlw	A'S'        ; Send processing statistics, ascii 'S'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'S'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'S'
	goto	Opt_S          ;  - if it was, jump
	movlw	A'M'        ; Set minimum range value, ascii 'M'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'M'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'M'
	goto	Opt_M          ;  - if it was, jump
	movlw	A'H'        ; Set raw data format to Hex, ascii 'H'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'H'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'H'
	goto	Opt_H          ;  - if it was, jump
	movlw	A'D'        ; Set raw data format to decimal, ascii 'D'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'D'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'D'
	goto	Opt_D          ;  - if it was, jump
	movlw	A'A'        ; Set angle format to BAM, ascii 'A'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'A'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'A'
	goto	Opt_A          ;  - if it was, jump
	movlw	A'G'        ; Set anble format to decimal degrees, ascii 'G'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'G'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'G'
	goto	Opt_G          ;  - if it was, jump
	movlw	A'C'        ; Computer control mode, ascii 'C'
	xorwf	tmp01,w        ; (tmp01) .XOR. 'C'  --> W
	btfsc	STATUS,Z       ; Test whether received byte was 'C'
	goto	Opt_C          ;  - if it was, jump
	; No valid menu selection found!
	movlw	NAK         ; ascii NAK --> W
	call	Txbyte      ;  send NAK to host
	goto	Host_01
;
Opt_ESC	    ; End computer control mode
	movlw	ACK         ; ascii ACK --> W
	call	Txbyte      ;  send ACK to host
	banksel	LATB        ; select bank 2
	bsf	LATB,3      ;  set BUSY line (RB3)
	bcf	mode,ccm    ; clear computer control mode flag
	goto	Host_01
;
Opt_A	    ; Use BAM format to report bearings to host
	movlw	ACK         ; ascii ACK --> W
	call	Txbyte      ;  send ACK to host
	banksel	LATB        ; select bank 2
	bsf	LATB,3      ;  set BUSY line (RB3)
	bsf	mode,baf    ; set bearing angle format to BAM
	goto	Host_01
;
Opt_B	    ; Send computed bearings to host
	btfsc	freg,rcb    ; check computed bearings flag
	bra	Opt_B1
	movlw	NAK         ; ascii NAK --> W
	call	Txbyte      ;  send NAK to host
	bra	Opt_B9
Opt_B1
	movlw	ACK         ; ascii ACK --> W
	call	Txbyte      ;  send ACK to host
	banksel	LATB        ; select bank 2
	bsf	LATB,3      ;  set BUSY line (RB3)
	movlb	0           ; 0 --> BSR, select bank 0
	btfsc	mode,baf    ; test bearing angle format
	bra	Opt_B5      ; - go if format is BAM
	movf	brng_a,w    ; amplitude bearing angle
	call	Txwasdeg    ;  send as decimal degrees
	movlw	A','        ; comma
	call	Txbyte
	movf	brng_x,w    ; xcorr bearing angle
	call	Txwasdeg    ;  send as decimal degrees
	bra	Opt_B8
Opt_B5
	movf	brng_a,w    ; amplitude bearing angle
	call	Txwashex    ;  send as Binary Angle Measure
	movlw	A','        ; comma
	call	Txbyte
	movf	brng_x,w    ; xc bearing angle
	call	Txwashex    ;  send as Binary Angle Measure
Opt_B8	
	call	Txnl        ; send CR, LF
Opt_B9
	clrf	freg        ; clear all flags
	goto	Host_01	
;
Opt_C	    ; Begin computer control mode
	movlw	ACK         ; ascii ACK --> W
	call	Txbyte      ;  send ACK to host
	banksel	LATB        ; select bank 2
	bsf	LATB,3      ;  set BUSY line (RB3)
	bsf	mode,ccm    ; set computer control mode flag
	goto	Host_01
;
Opt_D	    ; Use decimal CSV format to send raw data to host
	movlw	ACK         ; ascii ACK --> W
	call	Txbyte      ;  send ACK to host
	banksel	LATB        ; select bank 2
	bsf	LATB,3      ;  set BUSY line (RB3)
	bcf	mode,rdf    ; clear raw data format flag
	goto	Host_01
;
Opt_E	    ; Acquire 8-bit raw data, ignores min_rng
	movlw	ACK          ; ascii ACK --> W
	call	Txbyte       ;  send ACK to host
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
	call	Acq127x8x3   ; acquire raw data
	clrf	freg         ; clear all flags
	bsf	freg,r08     ;  set 8-bit data ready flag
	goto	Host_01
;
Opt_G	    ; Use decimal degrees format to report bearings to host
	movlw	ACK         ; ascii ACK --> W
	call	Txbyte      ;  send ACK to host
	banksel	LATB        ; select bank 2
	bsf	LATB,3      ;  set BUSY line (RB3)
	bcf	mode,baf    ; make bearing angle format decimal degrees
	goto	Host_01
;
Opt_H	    ; Use hex CSV format to send raw data to host
	movlw	ACK         ; ascii ACK --> W
	call	Txbyte      ;  send ACK to host
	banksel	LATB        ; select bank 2
	bsf	LATB,3      ;  set BUSY line (RB3)
	bsf	mode,rdf    ; set raw data format flag for hex
	goto	Host_01
;
Opt_M	    ; Set minimum dynamic range
	call	Rxbyte       ; get bcd_2
	andlw	0x01         ;  limit hundreds
	movwf	bcd_2        ;   to 1 max
	iorlw	0x30         ;    echo ascii number character 
	btfss	mode,ccm     ;     if not in computer control mode
	call	Txbyte       ;      echo character
	call	Rxbyte       ; get bcd_1
	andlw	0x0F         ;  limit to 
	movwf	bcd_1        ;   lower nibble
	iorlw	0x30         ;    echo ascii number character 
	btfss	mode,ccm     ;     if not in computer control mode
	call	Txbyte       ;      echo character
	call	Rxbyte       ; get bcd_0
	andlw	0x0F         ;  limit to
	movwf	bcd_0        ;   lower nibble
	iorlw	0x30         ;    echo ascii number character 
	btfss	mode,ccm     ;     if not in computer control mode
	call	Txbyte       ;      echo character
	movlw	ACK          ; ascii ACK --> W
	call	Txbyte       ;  send ACK to host
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
	movlb	0            ; select bank 0
	clrf	bcd_7        ; clear
	clrf	bcd_6        ;  upper
	clrf	bcd_5        ;   binary
	clrf	bcd_4        ;    coded
	clrf	bcd_3        ;     characters
	call	T_bcd2b      ; convert bcd to binary
	movf	ACb_0,w      ;  binary value --> W
	andlw	0x7F         ;   limit W to 127 maximum
	movwf	min_rng      ;    W --> min_rng
	clrf	freg         ; clear all flags
	goto	Host_01
;
Opt_P	    ; Acquire and process 8-bit data
	movlw	ACK          ; ascii ACK --> W
	call	Txbyte       ;  send ACK to host
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
Opt_P1
	call	Acq127x8x3   ; acquire raw data 
	call	Sort8x3      ; sort the data into channel buffers
	call	Chn_stat     ; find the gnd and rng for each channel
	btfss	freg,rok     ; test ranges okay flag
	bra	Opt_P1       ;  branch if flag cleared
	call	Chn_amp      ; compute amplitude bearing
	call	Chn_sign     ; convert data arrays to signed bytes
	call	X_corr       ; compute cross-correlation bearing
	bsf	freg,rcb     ; set computed bearings flag bit
	goto	Host_01
;
Opt_R	    ; Send raw data to host
	btfsc	freg,r08     ; check 8-bit data ready flag
	goto	OptR_80      ;  go if 8-bit data is ready
	btfsc	freg,r10     ; check 10-bit data ready flag
	goto	OptR_10      ;  go if 10-bit data is ready
	movlw	NAK          ; ascii NAK --> W
	call	Txbyte       ;  send NAK to host
	bra	Opt_R9
;
; send 10-bit raw data as 127 CSV lines. Data is sent in decimal
; format with leading zeros suppressed or hex as previously set.
OptR_10	; send 10-bit raw data
	movlw	ACK          ; ascii ACK --> W
	call	Txbyte       ;  send ACK to host
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
;
	movlb	0            ; 0 --> BSR, select bank 0
	movlw	D'127'       ; extract 127 samples (bytes) per channel
	movwf	lin_cnt      ; load line downcounter 
	movlw	low c3dat    ; low byte of acquisition buffer address
	movwf	FSR1L        ;  for 10-bit data --> FSR1L
	movlw	high c3dat   ; high byte of acquisition buffer address
	movwf	FSR1H        ;  for 10-bit data --> FSR1H
OptR_11
	clrf	ACx_0        ; channel 1 data will be in ACx
	clrf	ACy_0        ; channel 2 data will be in ACy
	clrf	ACz_0        ; channel 3 data will be in ACz
	movlw	5          ; five bytes from buffer
	movwf	byt_cnt      ; becomes three 10-bit values
OptR_12
	moviw	FSR1++     ; two bits of interleaved adc data
	movwf	s_hold       ; store it
	lslf	s_hold,f     ; discard b7 bit
	lslf	s_hold,f     ; b6, channel 3 bit --> C
	rlf	ACz_0,f      ;  rotate C into ls-bit of ACz_0
	rlf	ACz_1,f      ;   rotate msb of ACz_0 into ACz_1
	lslf	s_hold,f     ; b5, channel 2 bit --> C
	rlf	ACy_0,f      ;  rotate C into ls-bit of ACy_0
	rlf	ACy_1,f      ;   rotate msb of ACy_0 into ACy_1
	lslf	s_hold,f     ; b4, channel 1 bit --> C
	rlf	ACx_0,f      ;  rotate C into ls-bit of ACx_0
	rlf	ACx_1,f      ;   rotate msb of ACx_0 into ACx_1
	lslf	s_hold,f     ; discard b3 bit
	lslf	s_hold,f     ; b2, channel 3 bit --> C
	rlf	ACz_0,f      ;  rotate C into ls-bit of ACz_0
	rlf	ACz_1,f      ;   rotate msb of ACz_0 into ACz_1
	lslf	s_hold,f     ; b1, channel 2 bit --> C
	rlf	ACy_0,f      ;  rotate C into ls-bit of ACy_0
	rlf	ACy_1,f      ;   rotate msb of ACy_0 into ACy_1
	lslf	s_hold,f     ; b0, channel 1 bit --> C
	rlf	ACx_0,f      ;  rotate C into ls-bit of ACx_0
	rlf	ACx_1,f      ;   rotate msb of ACx_0 into ACx_1
	decfsz	byt_cnt,f    ; decrement byte counter
	goto	OptR_12      ; need to get all 10 bits
; send one line: (channel 1),(channel 2),(channel 3),CR,LF
	btfss	mode,rdf     ; test raw data format flag
	bra	Opt_R13d     ; - go if rdf = 0, decimal
	bra	Opt_R13h     ; - go if rdf = 1, hex
Opt_R13d    ; send raw data in decimal format
	clrf	ACb_2        ; 00 --> ACb_2
	movf	ACx_1,w      ; channel 1 msb
	movwf	ACb_1        ; ACx_1 --> ACb_1
	movf	ACx_0,w      ; channel 1 lsb
	movwf	ACb_0        ; ACx_0 --> ACb_0
	call	Txbasdec     ; send ACb in ascii decimal format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	clrf	ACb_2        ; 00 --> ACb_2
	movf	ACy_1,w      ; channel 2 msb
	movwf	ACb_1        ; ACy_1 --> ACb_1
	movf	ACy_0,w      ; channel 2 lsb
	movwf	ACb_0        ; ACy_0 --> ACb_0
	call	Txbasdec     ; send ACb in ascii decimal format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	clrf	ACb_2        ; 00 --> ACb_2
	movf	ACz_1,w      ; channel 3 msb
	movwf	ACb_1        ; ACz_1 --> ACb_1
	movf	ACz_0,w      ; channel 3 lsb
	movwf	ACb_0        ; ACz_0 --> ACb_0
	call	Txbasdec     ; send ACb in ascii decimal format
	goto	Opt_R17
Opt_R13h    ; send raw data in hex format
	movf	ACx_1,w      ; channel 1 msb
	call	Hex1asc      ; convert lsn to ascii-hex
	call	Txbyte       ; send in hex format
	movf	ACx_0,w      ; channel 1 lsb
	call	Txwashex     ; send in hex format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	movf	ACy_1,w      ; channel 2 msb
	call	Hex1asc      ; convert lsn to ascii-hex
	call	Txbyte       ; send in hex format
	movf	ACy_0,w      ; channel 2 lsb
	call	Txwashex     ; send in hex format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	movf	ACz_1,w      ; channel 3 msb
	call	Hex1asc      ; convert lsn to ascii-hex
	call	Txbyte       ; send in hex format
	movf	ACz_0,w      ; channel 3 lsb
	call	Txwashex     ; send in hex format
Opt_R17
	call	Txnl         ; end line
	decfsz	lin_cnt,f    ; decrement the line count
	goto	OptR_11      ; go if all 127 lines not done
	goto	Opt_R9
;
; send 8-bit raw data as 127 CSV lines. Data is sent in decimal
; format with leading zeros suppressed or hex as previously set.
OptR_80
	movlw	ACK          ; ascii ACK --> W
	call	Txbyte       ;  send ACK to host
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
;
	movlb	0            ; 0 --> BSR, select bank 0
	movlw	D'127'       ; extract 127 samples (bytes) per channel
	movwf	lin_cnt      ; load line downcounter 
	movlw	low abuff    ; low byte of acquisition buffer address
	movwf	FSR1L        ;  --> FSR1L
	movlw	high abuff   ; high byte of acquisition buffer address
	movwf	FSR1H        ;  --> FSR1H
OptR_81
	movlw	4          ; four bytes from abuff 
	movwf	byt_cnt      ; becomes three bytes  of adc data
OptR_82
	moviw	FSR1++     ; two bits of interleaved adc data
	movwf	s_hold       ; store it
	lslf	s_hold,f     ; discard b7 bit
	lslf	s_hold,f     ; b6, channel 3 bit --> C
	rlf	adc_3,f      ; rotate C into ls-bit of adc_3
	lslf	s_hold,f     ; b5, channel 2 bit --> C
	rlf	adc_2,f      ; rotate C into ls-bit of adc_2
	lslf	s_hold,f     ; b4, channel 1 bit --> C
	rlf	adc_1,f      ; rotate C into ls-bit of adc_1
	lslf	s_hold,f     ; discard b3 bit
	lslf	s_hold,f     ; b2, channel 3 bit --> C
	rlf	adc_3,f      ; rotate C into ls-bit of adc_3
	lslf	s_hold,f     ; b1, channel 2 bit --> C
	rlf	adc_2,f      ; rotate C into ls-bit of adc_2
	lslf	s_hold,f     ; b0, channel 1 bit --> C
	rlf	adc_1,f      ; rotate C into ls-bit of adc_1
	decfsz	byt_cnt,f    ; decrement byte counter
	goto	OptR_82      ; need to get all 8 bits
; send one line: (channel 1),(channel 2),(channel 3),CR,LF
	btfss	mode,rdf     ; test raw data format flag
	bra	Opt_R83d     ; - go if rdf = 0, decimal
	bra	Opt_R83h     ; - go if rdf = 1, hex
Opt_R83d    ; send raw data in decimal format
	movf	adc_1,w      ; adc_1 --> W
	call	Txwasdec     ;  send W in ascii decimal format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	movf	adc_2,w      ; adc_2 --> W
	call	Txwasdec     ;  send W in ascii decimal format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	movf	adc_3,w      ; adc_3 --> W
	call	Txwasdec     ;  send W in ascii decimal format
	goto	Opt_R87
Opt_R83h    ; send raw data in hex format
	movf	adc_1,w      ; adc_1 --> W
	call	Txwashex     ;  send W in hex format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	movf	adc_2,w      ; adc_2 --> W
	call	Txwashex     ;  send W in hex format
	movlw	A','         ; ascii comma --> W
	call	Txbyte       ;  send it
	movf	adc_3,w      ; adc_3 --> W
	call	Txwashex     ;  send W in hex format	
Opt_R87	
	call	Txnl         ; end line
	decfsz	lin_cnt,f    ; decrement the line count
	goto	OptR_81      ; go if all 127 lines not done
Opt_R9
	clrf	freg         ; clear all flags
	goto	Host_01
;
Opt_S	    ; Send processing statistics as decimal numbers
	btfsc	freg,rcb     ; check computed bearings flag
	bra	Opt_S1
	movlw	NAK          ; ascii NAK --> W
	call	Txbyte       ;  send NAK to host
	bra	Opt_S9
Opt_S1
	movlw	ACK          ; ascii ACK --> W
	call	Txbyte       ;  send ACK to host
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
	movf	min_rng,w    ; min_rng --> W
	call	Txwasdec     ;  send min_rng 
	movlw	A','         ;   ascii comma --> W
	call	Txbyte       ;    send it
	movf	rng_1,w      ; rng_1 --> W
	call	Txwasdec     ;  send rng_1 
	movlw	A','         ;   ascii comma --> W
	call	Txbyte       ;    send it
	movf	rng_2,w      ; rng_2 --> W
	call	Txwasdec     ;  send rng_2 
	movlw	A','         ;   ascii comma --> W
	call	Txbyte       ;    send it
	movf	rng_3,w      ; rng_3 --> W
	call	Txwasdec     ;  send rng_3 
	movlw	A','         ;   ascii comma --> W
	call	Txbyte       ;    send it
	movf	gnd_1,w      ; gnd_1 --> W
	call	Txwasdec     ;  send gnd_1 
	movlw	A','         ;   ascii comma --> W
	call	Txbyte       ;    send it
	movf	gnd_2,w      ; gnd_2 --> W
	call	Txwasdec     ;  send gnd_2 
	movlw	A','         ;   ascii comma --> W
	call	Txbyte       ;    send it
	movf	gnd_3,w      ; gnd_3 --> W
	call	Txwasdec     ;  send gnd_3 
	call	Txnl         ; end line
Opt_S9
	goto	Host_01
;
Opt_T	    ; Acquire 10-bit raw data, ignore min_rng
	movlw	ACK          ; ascii ACK --> W
	call	Txbyte       ;  send ACK to host
	banksel	LATB         ; select bank 2
	bsf	LATB,3       ;  set BUSY line (RB3)
	call	Acq127x10x3  ; acquire raw data
	clrf	freg         ; clear all flags
	bsf	freg,r10     ; set 10-bit data ready flag
	goto	Host_01
;	
;============================================================================
; Page 3, Tables and Messages
;============================================================================
;
	org	0x1800
; Table of angle values for amplitude bearing. Angles are 8-bit binary with
; the msb = 180 degrees and the lsb = 1.40625 degrees.
; Index into table = 0000QQAA where bits QQ are the trig quadrants 1-4 
; represented by 0-3, and AA are levels of the arctan ratio (16*Y/X) within
; any quadrant.
Tbl_amp
	retlw	0x00      ; index 00, 000 degrees
	retlw	0x16      ; index 01, 030 degrees
	retlw	0x2B      ; index 02, 060 degrees
	retlw	0x40      ; index 03, 090 degrees
	retlw	0x80      ; index 04, 180 degrees
	retlw	0x6B      ; index 05, 150 degrees
	retlw	0x56      ; index 06, 120 degrees
	retlw	0x40      ; index 07, 090 degrees
	retlw	0x80      ; index 08, 180 degrees
	retlw	0x96      ; index 09, 210 degrees
	retlw	0xAB      ; index 10, 240 degrees
	retlw	0xC0      ; index 11, 270 degrees
	retlw	0x00      ; index 12, 000 degrees
	retlw	0xEB      ; index 13, 330 degrees
	retlw	0xD6      ; index 14, 300 degrees
	retlw	0xC0      ; index 15, 270 degrees
; End of table
;============================================================================
	org	0x1814
; Table of baud values for UART.
Tbl_baud
	retlw	BD76800     ; index 00, RA3 and RA4 both jumpered
	retlw	BD38400     ; index 01, RA4 jumpered to Ground
	retlw	BD19200     ; index 10, RA3 jumpered to Ground
	retlw	BD9600      ; index 11, no jumpers
; End of table
;============================================================================
	org	0x1900
; Table of angle values for cross-correlation bearing. Angles are 8-bit
; binary with the msb = 180 degrees and the lsb = 1.40625 degrees.
; Index into table = 00112233 where 11,22,33 are bits representing the lag
; values from baselines 1-2, 2-3, 3-1 respectively. If the lag is negative
; the bits are 01, positive 10, and zero 00; 11 is invalid.
; Invalid lag combinations return a value of 0xFF.
Tbl_xcr
	retlw	0xFF      ; index 000000, 00, invalid lag combination
	retlw	0xFF      ; index 000001, 01, invalid lag combination
	retlw	0xFF      ; index 000010, 02, invalid lag combination
	retlw	0xFF      ; index 000011, 03, invalid index
	retlw	0xFF      ; index 000100, 04, invalid lag combination
	retlw	0xFF      ; index 000101, 05, invalid lag combination
	retlw	0x80      ; index 000110, 06, 180 degrees
	retlw	0xFF      ; index 000111, 07, invalid index
	retlw	0xFF      ; index 001000, 08, invalid lag combination
	retlw	0x00      ; index 001001, 09, 0 degrees
	retlw	0xFF      ; index 001010, 10, invalid lag combination
	retlw	0xFF      ; index 001011, 11, invalid index
	retlw	0xFF      ; index 001100, 12, invalid index
	retlw	0xFF      ; index 001101, 13, invalid index
	retlw	0xFF      ; index 001110, 14, invalid index
	retlw	0xFF      ; index 001111, 15, invalid index
	retlw	0xFF      ; index 010000, 16, invalid lag combination
	retlw	0xFF      ; index 010001, 17, invalid lag combination
	retlw	0xAB      ; index 010010, 18, 240 degrees
	retlw	0xFF      ; index 010011, 19, invalid index
	retlw	0xFF      ; index 010100, 20, invalid lag combination 
	retlw	0xFF      ; index 010101, 21, invalid lag combination
	retlw	0x96      ; index 010110, 22, sector 4, 210 degrees
	retlw	0xFF      ; index 010111, 23, invalid index
	retlw	0xD6      ; index 011000, 24, 300 degrees
	retlw	0xEB      ; index 011001, 25, sector 6, 330 degrees
	retlw	0xC0      ; index 011010, 26, sector 5, 270 degrees
	retlw	0xFF      ; index 011011, 27, invalid index
	retlw	0xFF      ; index 011100, 28, invalid index 
	retlw	0xFF      ; index 011101, 29, invalid index
	retlw	0xFF      ; index 011110, 30, invalid index 
	retlw	0xFF      ; index 011111, 31, invalid index
	retlw	0xFF      ; index 100000, 32, invalid lag combination
	retlw	0x2B      ; index 100001, 33, 60 degrees
	retlw	0xFF      ; index 100010, 34, invalid lag combination
	retlw	0xFF      ; index 100011, 35, invalid index
	retlw	0x56      ; index 100100, 36, 120 degrees
	retlw	0x40      ; index 100101, 37, sector 2, 90 degrees
	retlw	0x6B      ; index 100110, 38, sector 3, 150 degrees
	retlw	0xFF      ; index 100111, 39, invalid index
	retlw	0xFF      ; index 101000, 40, invalid lag combination
	retlw	0x16      ; index 101001, 41, sector 1, 30 degrees
	retlw	0xFF      ; index 101010, 42, invalid lag combination
; End of table
;============================================================================
	org	0x1C00
Menu1	dt	CR,LF,LF,"  ROBOEAR, firmware 1.05"
;	dt	CR,LF,"[@] Reset"
;	dt	CR,LF,"[ESC] End computer control mode"
	dt	CR,LF,"[E] Acquire one set of 8-bit data"
	dt	CR,LF,"[T] Acquire one set of 10-bit data"
	dt	CR,LF," [R] Send raw data in CSV format"
	dt	CR,LF,"[P] Acquire and process one set of 8-bit data"
	dt	CR,LF," [S] Send processing statistics"
	dt	CR,LF," [B] Send bearing angles"
	dt	CR,LF," [M] Set minimum channel range"
	dt	CR,LF,"[H] Use Hex values for raw data"
	dt	CR,LF,"[D] Use Decimal values for raw data"
	dt	CR,LF,"[A] Use BAM format for bearings"
	dt	CR,LF,"[G] Use decimal degrees format for bearings"
	dt	CR,LF,"[C] Computer control mode, ESCape to exit"
	dt	CR,LF," ? ",0
;
;============================================================================
;
	end

